@@ -22,9 +22,6 @@
 #ifdef _WIN32
 #include "boinc_win.h"
 #include "win_util.h"
-#ifdef _MSC_VER
-#define snprintf _snprintf
-#endif
 #ifndef STATUS_SUCCESS
 #define STATUS_SUCCESS                0x0         // may be in ntstatus.h
 #endif
@@ -69,6 +66,7 @@ using std::vector;
 #include "shmem.h"
 #include "str_replace.h"
 #include "str_util.h"
+#include "url.h"
 #include "util.h"
 
 #include "client_msgs.h"
@@ -190,7 +188,7 @@ bool ACTIVE_TASK::kill_all_children() {
 #endif
 #endif
 
-static void print_descendants(int pid, vector<int>desc, const char* where) {
+static void print_descendants(int pid, const vector<int>& desc, const char* where) {
     msg_printf(0, MSG_INFO, "%s: PID %d has %d descendants",
         where, pid, (int)desc.size()
     );
@@ -238,10 +236,10 @@ int ACTIVE_TASK::request_abort() {
 #ifdef _WIN32
 static void kill_app_process(int pid, bool will_restart) {
     int retval = 0;
-    retval = kill_program(pid, will_restart?0:EXIT_ABORTED_BY_CLIENT);
+    retval = kill_process_with_status(pid, will_restart?0:EXIT_ABORTED_BY_CLIENT);
     if (retval && log_flags.task_debug) {
         msg_printf(0, MSG_INFO,
-            "[task] kill_program(%d) failed: %s",
+            "[task] kill_process_with_status(%d) failed: %s",
             pid, boincerror(retval)
         );
     }
@@ -259,10 +257,10 @@ static void kill_app_process(int pid, bool) {
             );
         }
     } else {
-        retval = kill_program(pid);
+        retval = kill_process(pid);
         if (retval && log_flags.task_debug) {
             msg_printf(0, MSG_INFO,
-                "[task] kill_program(%d) failed: %s",
+                "[task] kill_process(%d) failed: %s",
                 pid, strerror(errno)
             );
         }
@@ -502,7 +500,7 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
             char szError[1024];
             set_task_state(PROCESS_EXITED, "handle_exited_app");
             snprintf(err_msg, sizeof(err_msg),
-                "%s - exit code %d (0x%x)",
+                "%s - exit code %lu (0x%x)",
                 windows_format_error_string(exit_code, szError, sizeof(szError)),
                 exit_code, exit_code
             );
@@ -611,7 +609,9 @@ void ACTIVE_TASK::handle_exited_app(int stat) {
                 "read_stderr_file(): %s", boincerror(retval)
             );
         }
-        client_clean_out_dir(slot_dir, "handle_exited_app()");
+        if (!wup->project->app_test) {
+            client_clean_out_dir(slot_dir, "handle_exited_app()");
+        }
         clear_schedule_backoffs(this);
             // clear scheduling backoffs of jobs waiting for GPU
     }
@@ -632,7 +632,7 @@ bool ACTIVE_TASK::finish_file_present(int &exit_code) {
 
     exit_code = 0;
 
-    sprintf(path, "%s/%s", slot_dir, BOINC_FINISH_CALLED_FILE);
+    snprintf(path, sizeof(path), "%s/%s", slot_dir, BOINC_FINISH_CALLED_FILE);
     FILE* f = boinc_fopen(path, "r");
     if (!f) return false;
     char* p = fgets(buf, sizeof(buf), f);
@@ -644,11 +644,12 @@ bool ACTIVE_TASK::finish_file_present(int &exit_code) {
     }
     p = fgets(buf, sizeof(buf), f);
     if (p && strlen(buf)) {
-        fgets(buf2, sizeof(buf2), f);
-        msg_printf(result->project,
-            strstr(buf2, "notice")?MSG_USER_ALERT:MSG_INFO,
-            "Message from task: %s", buf
-        );
+        if (fgets(buf2, sizeof(buf2), f)) {
+            msg_printf(result->project,
+                strstr(buf2, "notice")?MSG_USER_ALERT:MSG_INFO,
+                "Message from task: %s", buf
+            );
+        }
     }
     fclose(f);
     return true;
@@ -658,7 +659,7 @@ bool ACTIVE_TASK::temporary_exit_file_present(
     double& x, char* buf, bool& is_notice
 ) {
     char path[MAXPATHLEN], buf2[256];
-    sprintf(path, "%s/%s", slot_dir, TEMPORARY_EXIT_FILE);
+    snprintf(path, sizeof(path), "%s/%s", slot_dir, TEMPORARY_EXIT_FILE);
     FILE* f = boinc_fopen(path, "r");
     if (!f) return false;
     strcpy(buf, "");
@@ -669,8 +670,14 @@ bool ACTIVE_TASK::temporary_exit_file_present(
     } else {
         x = y;
     }
-    (void) fgets(buf, 256, f);     // read the \n
-    (void) fgets(buf, 256, f);
+    char *p = fgets(buf, 256, f);     // read the \n
+    if (p) {
+        p = fgets(buf, 256, f);
+    }
+    if (p == NULL) {
+        fclose(f);
+        return false;
+    }
     strip_whitespace(buf);
     is_notice = false;
     if (fgets(buf2, 256, f)) {
@@ -720,12 +727,19 @@ void ACTIVE_TASK_SET::send_heartbeats() {
         if (gstate.network_suspended) {
             safe_strcat(buf, "<network_suspended/>");
         }
+        if (atp->sporadic_ca_state != CA_NONE) {
+            char buf2[256];
+            sprintf(buf2, "<sporadic_ca>%d</sporadic_ca>",
+                atp->sporadic_ca_state
+            );
+            safe_strcat(buf, buf2);
+        }
         bool sent = atp->app_client_shm.shm->heartbeat.send_msg(buf);
         if (log_flags.heartbeat_debug) {
             if (sent) {
                 msg_printf(atp->result->project, MSG_INFO,
-                    "[heartbeat] Heartbeat sent to task %s",
-                    atp->result->name
+                    "[heartbeat] Heartbeat sent to task %s: %s",
+                    atp->result->name, buf
                 );
             } else {
                 msg_printf(atp->result->project, MSG_INFO,
@@ -882,8 +896,8 @@ bool ACTIVE_TASK_SET::check_rsc_limits_exceeded() {
     for (i=0; i<active_tasks.size(); i++) {
         atp = active_tasks[i];
         if (atp->task_state() != PROCESS_EXECUTING) continue;
-        if (!atp->result->non_cpu_intensive() && (atp->elapsed_time > atp->max_elapsed_time)) {
-            sprintf(buf, "exceeded elapsed time limit %.2f (%.2fG/%.2fG)",
+        if (!atp->always_run() && (atp->elapsed_time > atp->max_elapsed_time)) {
+            snprintf(buf, sizeof(buf), "exceeded elapsed time limit %.2f (%.2fG/%.2fG)",
                 atp->max_elapsed_time,
                 atp->result->wup->rsc_fpops_bound/1e9,
                 atp->result->avp->flops/1e9
@@ -903,7 +917,7 @@ bool ACTIVE_TASK_SET::check_rsc_limits_exceeded() {
         // accurate bounds.
         //
         if (atp->procinfo.working_set_size_smoothed > atp->max_mem_usage) {
-            sprintf(buf, "working set size > workunit.rsc_memory_bound: %.2fMB > %.2fMB",
+            snprintf(buf, sizeof(buf), "working set size > workunit.rsc_memory_bound: %.2fMB > %.2fMB",
                 atp->procinfo.working_set_size_smoothed/MEGA, atp->max_mem_usage/MEGA
             );
             msg_printf(atp->result->project, MSG_INFO,
@@ -916,7 +930,7 @@ bool ACTIVE_TASK_SET::check_rsc_limits_exceeded() {
         }
 #endif
         if (atp->procinfo.working_set_size_smoothed > max_ram) {
-            sprintf(buf, "working set size > client RAM limit: %.2fMB > %.2fMB",
+            snprintf(buf, sizeof(buf), "working set size > client RAM limit: %.2fMB > %.2fMB",
                 atp->procinfo.working_set_size_smoothed/MEGA, max_ram/MEGA
             );
             msg_printf(atp->result->project, MSG_INFO,
@@ -936,7 +950,7 @@ bool ACTIVE_TASK_SET::check_rsc_limits_exceeded() {
 
         // don't count RAM usage of non-CPU-intensive jobs
         //
-        if (!atp->result->non_cpu_intensive()) {
+        if (!atp->non_cpu_intensive()) {
             ram_left -= atp->procinfo.working_set_size_smoothed;
         }
     }
@@ -982,7 +996,7 @@ int ACTIVE_TASK::read_stderr_file() {
     // it's unlikely that more than that will be useful
     //
     int max_len = 63*1024;
-    sprintf(path, "%s/%s", slot_dir, STDERR_FILE);
+    snprintf(path, sizeof(path), "%s/%s", slot_dir, STDERR_FILE);
     if (!boinc_file_exists(path)) return 0;
     int retval  = read_file_malloc(
         path, buf1, max_len, !cc_config.stderr_head
@@ -1199,7 +1213,7 @@ void ACTIVE_TASK_SET::suspend_all(int reason) {
 
         // special cases for non-CPU-intensive apps
         //
-        if (atp->result->non_cpu_intensive()) {
+        if (atp->non_cpu_intensive()) {
             if (cc_config.dont_suspend_nci) {
                 continue;
             }
@@ -1211,7 +1225,7 @@ void ACTIVE_TASK_SET::suspend_all(int reason) {
         // handle CPU throttling separately
         //
         if (reason == SUSPEND_REASON_CPU_THROTTLE) {
-            if (atp->result->dont_throttle()) continue;
+            if (atp->dont_throttle()) continue;
             atp->preempt(REMOVE_NEVER, reason);
             continue;
         }
@@ -1239,7 +1253,7 @@ void ACTIVE_TASK_SET::suspend_all(int reason) {
             // which uses a lot of CPU.
             // Avoid going into a preemption loop.
             //
-            if (atp->result->non_cpu_intensive()) break;
+            if (atp->always_run()) break;
             atp->preempt(REMOVE_NEVER);
             break;
         case SUSPEND_REASON_BATTERY_OVERHEATED:
@@ -1383,7 +1397,7 @@ void ACTIVE_TASK::send_network_available() {
 bool ACTIVE_TASK::get_app_status_msg() {
     char msg_buf[MSG_CHANNEL_SIZE];
     double fd;
-    int other_pid;
+    int other_pid, i;
     double dtemp;
     static double last_msg_time=0;
 
@@ -1453,6 +1467,9 @@ bool ACTIVE_TASK::get_app_status_msg() {
         other_pids.clear();
         other_pids.push_back(other_pid);
     }
+    if (parse_int(msg_buf, "<sporadic_ac>", i)) {
+        sporadic_ac_state = (SPORADIC_AC_STATE)i;
+    }
     if (current_cpu_time < 0) {
         msg_printf(result->project, MSG_INFO,
             "app reporting negative CPU: %f", current_cpu_time
@@ -1541,7 +1558,7 @@ void ACTIVE_TASK_SET::get_msgs() {
     last_time = gstate.now;
 
     double et_diff = delta_t;
-    double et_diff_throttle = delta_t * gstate.global_prefs.cpu_usage_limit/100;
+    double et_diff_throttle = delta_t * gstate.current_cpu_usage_limit()/100;
 
     for (i=0; i<active_tasks.size(); i++) {
         atp = active_tasks[i];
@@ -1555,7 +1572,7 @@ void ACTIVE_TASK_SET::get_msgs() {
         if (atp->get_app_status_msg()) {
             if (old_time != atp->checkpoint_cpu_time) {
                 char buf[512];
-                sprintf(buf, "%s checkpointed", atp->result->name);
+                snprintf(buf, sizeof(buf), "%s checkpointed", atp->result->name);
                 if (atp->overdue_checkpoint) {
                     gstate.request_schedule_cpus(buf);
                 }
@@ -1584,7 +1601,7 @@ void ACTIVE_TASK_SET::get_msgs() {
 //
 void ACTIVE_TASK::write_task_state_file() {
     char path[MAXPATHLEN];
-    sprintf(path, "%s/%s", slot_dir, TASK_STATE_FILENAME);
+    snprintf(path, sizeof(path), "%s/%s", slot_dir, TASK_STATE_FILENAME);
     FILE* f = boinc_fopen(path, "w");
     if (!f) return;
     fprintf(f,
@@ -1615,12 +1632,15 @@ void ACTIVE_TASK::write_task_state_file() {
 //
 void ACTIVE_TASK::read_task_state_file() {
     char buf[4096], path[MAXPATHLEN], s[1024];
-    sprintf(path, "%s/%s", slot_dir, TASK_STATE_FILENAME);
+    snprintf(path, sizeof(path), "%s/%s", slot_dir, TASK_STATE_FILENAME);
     FILE* f = boinc_fopen(path, "r");
     if (!f) return;
     buf[0] = 0;
-    (void) fread(buf, 1, 4096, f);
+    size_t n = fread(buf, 1, 4096, f);
     fclose(f);
+    if (n == 0) {
+        return;
+    }
     buf[4095] = 0;
     double x;
     // TODO: use XML parser
@@ -1633,7 +1653,7 @@ void ACTIVE_TASK::read_task_state_file() {
         );
         return;
     }
-    if (strcmp(s, result->project->master_url)) {
+    if (!urls_match(s, result->project->master_url)) {
         msg_printf(wup->project, MSG_INTERNAL_ERROR,
             "wrong project URL in task state file"
         );
