@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2017 University of California
+// Copyright (C) 2023 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -34,17 +34,21 @@
 #endif
 
 #ifdef __APPLE__                            // If Mac BOINC Manager
+#include "mac_branding.h"
+
 bool IsUserInGroup(char* groupName);
 #endif
 
 static int CheckNestedDirectories(
-    char * basepath, int depth, 
-    int use_sandbox, int isManager, 
-    char * path_to_error,
+    char * basepath, int depth,
+    int use_sandbox, int isManager,
+    bool isSlotDir, char * path_to_error,
     int len
 );
 
 #if (! defined(__WXMAC__) && ! defined(_MAC_INSTALLER))
+#include "sandbox.h"
+
 static char * PersistentFGets(char *buf, size_t buflen, FILE *f);
 static void GetPathToThisProcess(char* outbuf, size_t maxLen);
 #endif
@@ -72,26 +76,21 @@ int use_sandbox, int isManager, char* path_to_error, int len
 ) {
     passwd              *pw;
     group               *grp;
-    gid_t               egid;
-    uid_t               euid;
     char                dir_path[MAXPATHLEN], full_path[MAXPATHLEN];
     struct stat         sbuf;
     int                 retval;
     int                 useFakeProjectUserAndGroup = 0;
-
-#define NUMBRANDS 3
-
-char *saverName[NUMBRANDS];
-
-saverName[0] = "BOINCSaver";
-saverName[1] = "GridRepublic";
-saverName[2] = "Progress Thru Processors";
+    int                 isMacInstaller = 0;
 
     useFakeProjectUserAndGroup = ! use_sandbox;
 #ifdef _DEBUG
 #ifdef DEBUG_WITH_FAKE_PROJECT_USER_AND_GROUP
     useFakeProjectUserAndGroup = 1;
 #endif
+#ifdef _MAC_INSTALLER
+    isMacInstaller = 1;
+#endif
+
 #if (defined(__APPLE__) &&  ! defined(_MAC_INSTALLER))
     // Debugging Mac client or Mac BOINC Manager
     // Normally, the Mac Development (Debug) builds do not define SANDBOX, so
@@ -99,7 +98,7 @@ saverName[2] = "Progress Thru Processors";
     // or LLDB on sandbox-specific code, as long as the code is run as the
     // current user (i.e., not as boinc_master or boinc_project), and the
     // current user is a member of both groups boinc_master and boinc_project.
-    // Note that this newer approach supersedes the old one described and 
+    // Note that this newer approach supersedes the old one described and
     // implemented below. But since it has not been thoroughly tested, I have
     // not removed the old code.
     if (IsUserInGroup(REAL_BOINC_MASTER_NAME) && IsUserInGroup(REAL_BOINC_PROJECT_NAME)) {
@@ -110,10 +109,10 @@ saverName[2] = "Progress Thru Processors";
 #endif
 #endif      // _DEBUG
 
-// GDB can't attach to applications which are running as a diferent user or group so
-//  it ignores the S_ISUID and S_ISGID permisison bits when launching an application.
+// GDB can't attach to applications which are running as a different user or group so
+//  it ignores the S_ISUID and S_ISGID permission bits when launching an application.
 // To work around this, and to allow testing the uninstalled Deployment version, we
-//  assume that the BOINC Client has the correct user and group. 
+//  assume that the BOINC Client has the correct user and group.
 // We must get the BOINC Client's user and group differently depending on whether we
 //  were called from the Manager or from the Client
 
@@ -159,33 +158,37 @@ saverName[2] = "Progress Thru Processors";
         boinc_master_gid = getegid();
     }
 
-#ifdef _MAC_INSTALLER
-   // Require absolute owner and group boinc_master:boinc_master
-    strlcpy(boinc_master_user_name, REAL_BOINC_MASTER_NAME, sizeof(boinc_master_user_name));
-    pw = getpwnam(boinc_master_user_name);
-    if (pw == NULL)
-        return -1006;      // User boinc_master does not exist
-    boinc_master_uid = pw->pw_uid;
+    if ((!useFakeProjectUserAndGroup) || isMacInstaller) {
+       // Require absolute owner and group boinc_master:boinc_master
+        strlcpy(boinc_master_user_name, REAL_BOINC_MASTER_NAME, sizeof(boinc_master_user_name));
+        pw = getpwnam(boinc_master_user_name);
+        if (pw == NULL)
+            return -1006;      // User boinc_master does not exist
+        boinc_master_uid = pw->pw_uid;
 
-    strlcpy(boinc_master_group_name, REAL_BOINC_MASTER_NAME, sizeof(boinc_master_group_name));
-    grp = getgrnam(boinc_master_group_name);
-    if (grp == NULL)
-        return -1007;                // Group boinc_master does not exist
-    boinc_master_gid = grp->gr_gid;
+        strlcpy(boinc_master_group_name, REAL_BOINC_MASTER_NAME, sizeof(boinc_master_group_name));
+        grp = getgrnam(boinc_master_group_name);
+        if (grp == NULL)
+            return -1007;                // Group boinc_master does not exist
+        boinc_master_gid = grp->gr_gid;
 
-#else   // Use current user and group (see comment above)
+        // A MacOS update sometimes changes the PrimaryGroupID of user boinc_master to staff (20).
+        if (pw->pw_gid != grp->gr_gid) {
+            return -1301;
+        }
+    } else {    // Use current user and group (see comment above)
 
-    pw = getpwuid(boinc_master_uid);
-    if (pw == NULL)
-        return -1008;      // Should never happen
-    strlcpy(boinc_master_user_name, pw->pw_name, sizeof(boinc_master_user_name));
+        pw = getpwuid(boinc_master_uid);
+        if (pw == NULL)
+            return -1008;      // Should never happen
+        strlcpy(boinc_master_user_name, pw->pw_name, sizeof(boinc_master_user_name));
 
-    grp = getgrgid(boinc_master_gid);
-    if (grp == NULL)
-        return -1009;
-    strlcpy(boinc_master_group_name, grp->gr_name, sizeof(boinc_master_group_name));
+        grp = getgrgid(boinc_master_gid);
+        if (grp == NULL)
+            return -1009;
+        strlcpy(boinc_master_group_name, grp->gr_name, sizeof(boinc_master_group_name));
 
-#endif
+    }
 
     if (useFakeProjectUserAndGroup) {
         // For easier debugging of project applications
@@ -205,6 +208,11 @@ saverName[2] = "Progress Thru Processors";
         if (grp == NULL)
             return -1011;                // Group boinc_project does not exist
         boinc_project_gid = grp->gr_gid;
+
+        // A MacOS update sometimes changes the PrimaryGroupID of user boinc_project to staff (20).
+        if (pw->pw_gid != grp->gr_gid) {
+            return -1302;
+        }
     }
 
 #if 0   // Manager is no longer setgid
@@ -223,12 +231,12 @@ saverName[2] = "Progress Thru Processors";
         strlcat(full_path, p+1, sizeof(full_path));
         p = strrchr(full_path, '.');         // Strip off  bundle extension (".app")
         if (p)
-            *p = '\0'; 
+            *p = '\0';
 
         retval = lstat(full_path, &sbuf);
         if (retval)
             return -1013;          // Should never happen
-        
+
         if (sbuf.st_gid != boinc_master_gid)
             return -1014;
 
@@ -251,7 +259,7 @@ saverName[2] = "Progress Thru Processors";
         retval = stat(full_path, &sbuf);
         if (retval)
             return -1016;          // Should never happen
-        
+
         if (sbuf.st_gid != boinc_master_gid)
             return -1017;
 
@@ -261,15 +269,15 @@ saverName[2] = "Progress Thru Processors";
 
 #if (defined(__WXMAC__) || defined(_MAC_INSTALLER)) // If Mac BOINC Manager or installer
         // Version 6 screensaver has its own embedded switcher application, but older versions don't.
-        // We don't allow unauthorized users to run the standard switcher application in the BOINC 
-        // Data directory because they could use it to run as user & group boinc_project and damage 
-        // project files.  The screensaver's gfx_switcher application has very limited functionality 
+        // We don't allow unauthorized users to run the standard switcher application in the BOINC
+        // Data directory because they could use it to run as user & group boinc_project and damage
+        // project files.  The screensaver's gfx_switcher application has very limited functionality
         // to avoid this risk.
 
         if (use_sandbox) {
             for (int i=0; i<NUMBRANDS; i++) {
                 // Does gfx_switcher exist in screensaver bundle?
-                sprintf(full_path, "/Library/Screen Savers/%s.saver/Contents/Resources/gfx_switcher", saverName[i]);
+                snprintf(full_path, sizeof(full_path), "/Library/Screen Savers/%s.saver/Contents/Resources/gfx_switcher", saverName[i]);
                 retval = stat(full_path, &sbuf);
                 if (! retval) {
 #ifdef _DEBUG
@@ -299,12 +307,13 @@ saverName[2] = "Progress Thru Processors";
 
 //    rgid = getgid();
 //    ruid = getuid();
-    egid = getegid();
-    euid = geteuid();
 
 #ifdef _MAC_INSTALLER
     strlcpy(dir_path, dataPath, sizeof(dir_path));  // Installer
 #else       // _MAC_INSTALLER
+    gid_t               egid = getegid();;
+    uid_t               euid = geteuid();;
+
     getcwd(dir_path, sizeof(dir_path));             // Client or Manager
 
     if (! isManager) {                                   // If BOINC Client
@@ -322,15 +331,15 @@ saverName[2] = "Progress Thru Processors";
 
     if (use_sandbox) {
 
-        // The top-level BOINC Data directory can have a different user if created by the Manager, 
+        // The top-level BOINC Data directory can have a different user if created by the Manager,
         // but it should always have group boinc_master.
         if (sbuf.st_gid != boinc_master_gid)
             return -1022;
-        
+
         // The top-level BOINC Data directory should have permission 771 or 571
         if ((sbuf.st_mode & 0577) != 0571)
             return -1023;
-            
+
     } else {
 
         if (sbuf.st_uid != boinc_master_uid)
@@ -341,7 +350,7 @@ saverName[2] = "Progress Thru Processors";
     snprintf(full_path, sizeof(full_path), "%s/%s", dir_path, PROJECTS_DIR);
     retval = stat(full_path, &sbuf);
     if (!retval) {
-        // Client can create projects directory if it does not yet exist.  
+        // Client can create projects directory if it does not yet exist.
         //
         if (use_sandbox) {
             if (sbuf.st_gid != boinc_project_gid)
@@ -350,19 +359,19 @@ saverName[2] = "Progress Thru Processors";
         if ((sbuf.st_mode & 0777) != 0770)
             return -1025;
         }
-        
+
         if (sbuf.st_uid != boinc_master_uid)
             return -1026;
 
         // Step through project directories
-        retval = CheckNestedDirectories(full_path, 1, use_sandbox, isManager, path_to_error, len);
+        retval = CheckNestedDirectories(full_path, 1, use_sandbox, isManager, false, path_to_error, len);
         if (retval)
             return retval;
     }
 
     snprintf(full_path, sizeof(full_path), "%s/%s", dir_path, SLOTS_DIR);
     retval = stat(full_path, &sbuf);
-    if (! retval) {                 // Client can create slots directory if it does not yet exist.  
+    if (! retval) {                 // Client can create slots directory if it does not yet exist.
        if (use_sandbox) {
             if (sbuf.st_gid != boinc_project_gid)
                 return -1027;
@@ -370,12 +379,12 @@ saverName[2] = "Progress Thru Processors";
             if ((sbuf.st_mode & 0777) != 0770)
                 return -1028;
         }
-        
+
         if (sbuf.st_uid != boinc_master_uid)
             return -1029;
 
         // Step through slot directories
-        retval = CheckNestedDirectories(full_path, 1, use_sandbox, isManager, path_to_error, len);
+        retval = CheckNestedDirectories(full_path, 1, use_sandbox, isManager, true, path_to_error, len);
         if (retval)
             return retval;
     }
@@ -384,7 +393,7 @@ saverName[2] = "Progress Thru Processors";
         "%s/%s", dir_path, GUI_RPC_PASSWD_FILE
     );
     retval = stat(full_path, &sbuf);
-    if (! retval) {                 // Client can create RPC password file if it does not yet exist.  
+    if (! retval) {                 // Client can create RPC password file if it does not yet exist.
         if (use_sandbox) {
             if (sbuf.st_gid != boinc_master_gid)
                 return -1030;
@@ -395,7 +404,7 @@ saverName[2] = "Progress Thru Processors";
             if ((sbuf.st_mode & 0717) != 0600)
                 return -1032;
         }
-        
+
         if (sbuf.st_uid != boinc_master_uid)
             return -1031;
     }
@@ -405,7 +414,7 @@ saverName[2] = "Progress Thru Processors";
         retval = stat(full_path, &sbuf);
         if (retval)
             return -1033;
-          
+
         if (sbuf.st_gid != boinc_master_gid)
             return -1034;
 
@@ -420,7 +429,7 @@ saverName[2] = "Progress Thru Processors";
         retval = stat(full_path, &sbuf);
         if (retval)
             return -1037;
-        
+
         if (sbuf.st_gid != boinc_master_gid)
             return -1038;
 
@@ -436,14 +445,14 @@ saverName[2] = "Progress Thru Processors";
         retval = stat(full_path, &sbuf);
         if (retval)
             return -1041;
-        
-        if (sbuf.st_gid != boinc_project_gid)
+
+        if (sbuf.st_gid != boinc_master_gid)
             return -1042;
 
-        if (sbuf.st_uid != boinc_master_uid)
+        if (sbuf.st_uid != 0)   // root
             return -1043;
 
-        if ((sbuf.st_mode & 07777) != 02500)
+        if ((sbuf.st_mode & 07777) != 04050)
             return -1044;
 
 #ifdef __APPLE__
@@ -453,7 +462,7 @@ saverName[2] = "Progress Thru Processors";
         retval = stat(full_path, &sbuf);
         if (retval)
             return -1045;
-        
+
         if (sbuf.st_gid != boinc_master_gid)
             return -1046;
 
@@ -480,16 +489,17 @@ saverName[2] = "Progress Thru Processors";
             if ((sbuf.st_mode & 0777) != 0664)
                 return -1053;
         }   // Screensaver config file ss_config.xml exists
-        
+
     }       // if (use_sandbox)
-    
+
     return 0;
 }
 
 
 static int CheckNestedDirectories(
     char * basepath, int depth,
-    int use_sandbox, int isManager, 
+    int use_sandbox, int isManager,
+    bool isSlotDir,
     char * path_to_error, int len
 ) {
     int             isDirectory;
@@ -502,10 +512,10 @@ static int CheckNestedDirectories(
 
     dirp = opendir(basepath);
     if (dirp == NULL) {
-        // Ideally, all project-created subdirectories under project or slot 
-        // directoriesshould have read-by-group and execute-by-group permission 
-        // bits set, but some don't.  If these permission bits are missing, the 
-        // project applications will run OK but we can't access the contents of 
+        // Ideally, all project-created subdirectories under project or slot
+        // directoriesshould have read-by-group and execute-by-group permission
+        // bits set, but some don't.  If these permission bits are missing, the
+        // project applications will run OK but we can't access the contents of
         // the subdirectory to check them.
         strlcpy(full_path, basepath, sizeof(full_path));
         if ((depth > 1) && (errno == EACCES)) {
@@ -519,7 +529,7 @@ static int CheckNestedDirectories(
         dp = readdir(dirp);
         if (dp == NULL)
             break;                  // End of list
-            
+
         if (dp->d_name[0] == '.')
             continue;               // Ignore names beginning with '.'
 
@@ -534,27 +544,52 @@ static int CheckNestedDirectories(
             if (depth > 1)  {
                 // files and subdirectories created by projects may have owner boinc_master or boinc_project
                 if ( (sbuf.st_uid != boinc_master_uid) && (sbuf.st_uid != boinc_project_uid) ) {
-                    retval = -1202;
-                    break;
-                }
-            } else {
-                // project & slot directories (projects/setiathome.berkeley.edu, slots/0 etc.) 
+                    if (!isSlotDir) {
+                            retval = -1202;
+                            break;
+                    } else {    // Slots
+                        // Graphics apps called by screensaver or Manager (via Show
+                        // Graphics button) now write files in their slot directory
+                        // as the logged in user, not boinc_master. In most cases,
+                        // the manager or screensaver will tell the client to fix
+                        // all ownerships in that slot directory, but a crash can
+                        // prevent that. As a backup strategy, we ignore ownership
+                        // of files in slot directories under the Manager, and fix
+                        // them in the client.
+#if (! defined(__WXMAC__) && ! defined(_MAC_INSTALLER))
+                        char* s = strstr(full_path, "/slots/");
+                        if (s) {
+                            s = strchr(s+1, '/');
+                            if (s) {
+                                int slot = atoi(s+1);
+                                fix_slot_owners(slot);  // client
+                            }
+                        }
+
+#elif defined(_MAC_INSTALLER)
+                            retval = -1202;
+                            break;
+#endif
+                        }   // isSlotDir
+                    }       // bad uid
+                } else {    // depth == 1
+                // project & slot directories (projects/setiathome.berkeley.edu, slots/0 etc.)
                 // must have owner boinc_master
                 if (sbuf.st_uid != boinc_master_uid) {
                     retval = -1202;
                     break;
                 }
             }
-        
+
             if (use_sandbox) {
                     if (sbuf.st_gid != boinc_project_gid) {
                         retval = -1201;
                         break;
                     }
-                
+
                 if (isDirectory) {
                     if (depth == 1) {
-                    // project & slot directories (projects/setiathome.berkeley.edu, slots/0 etc.) 
+                    // project & slot directories (projects/setiathome.berkeley.edu, slots/0 etc.)
                     // must be readable & executable by other
                         if ((sbuf.st_mode & 0777) != 0775) {
                             retval = -1203;
@@ -579,16 +614,16 @@ static int CheckNestedDirectories(
                 }
             }       // if (use_sandbox)
         }           // if (!S_ISLNK(sbuf.st_mode))
-        
+
         if (isDirectory && !S_ISLNK(sbuf.st_mode)) {
             if (use_sandbox && (depth > 1)) {
-#if 0   // No longer check project-created subdirectories under project or slot directories 
+#if 0   // No longer check project-created subdirectories under project or slot directories
         // because we have not told projects these must be readable and executable by group
                 if ((! isManager) && (sbuf.st_uid != boinc_master_uid))
 #endif
                     continue;       // Client can't check subdirectories owned by boinc_project
             }
-            retval = CheckNestedDirectories(full_path, depth + 1, use_sandbox, isManager, path_to_error, len);
+            retval = CheckNestedDirectories(full_path, depth + 1, use_sandbox, isManager, isSlotDir, path_to_error, len);
             if (retval)
                 break;
         }
@@ -598,7 +633,7 @@ static int CheckNestedDirectories(
     if (dirp) {
         closedir(dirp);
     }
-    
+
     if (retval && !errShown) {
         fprintf(stderr, "Permissions error %d at %s\n", retval, full_path);
         if (path_to_error) {
@@ -637,12 +672,12 @@ static void GetPathToThisProcess(char* outbuf, size_t maxLen) {
     pid_t aPID = getpid();
 
     *outbuf = '\0';
-    
-    sprintf(buf, "ps -xwo command -p %d", (int)aPID);
+
+    snprintf(buf, sizeof(buf), "ps -xwo command -p %d", (int)aPID);
     f = popen(buf, "r");
     if (f == NULL)
         return;
-    
+
     PersistentFGets (outbuf, maxLen, f);      // Discard header line
     PersistentFGets (outbuf, maxLen, f);
     pclose(f);
@@ -651,7 +686,7 @@ static void GetPathToThisProcess(char* outbuf, size_t maxLen) {
     p = strchr(outbuf, '\n');
     if (p)
         *p = '\0';
-    
+
     // Strip off any arguments
     p = strstr(outbuf, " -");
     q = p;
@@ -662,7 +697,7 @@ static void GetPathToThisProcess(char* outbuf, size_t maxLen) {
                 break;
         }
     }
-    
+
     if (q)
         *q = '\0';
 }
@@ -683,7 +718,7 @@ bool IsUserInGroup(char* groupName) {
             return true;                // User's primary group is boinc_master
         }
 
-        // On some systems with Automatic Login set, getlogin() returns "root" for a 
+        // On some systems with Automatic Login set, getlogin() returns "root" for a
         // time after the system is first booted, so we check "USER" environment variable.
         userName = getenv("USER");
         if (userName) {
