@@ -21,9 +21,6 @@
 
 #ifdef _WIN32
 #include "boinc_win.h"
-#ifdef _MSC_VER
-#define snprintf _snprintf
-#endif
 #else
 #ifdef __APPLE__
 // Suppress obsolete warning when building for OS 10.3.9
@@ -112,6 +109,13 @@ static bool is_intel(char* vendor) {
     return false;
 }
 
+#ifdef __APPLE__
+static bool is_apple(char* vendor) {
+    if (strcasestr(vendor, "apple")) return true;
+    return false;
+}
+#endif
+
 // If "loose", tolerate small diff
 //
 static int opencl_compare(OPENCL_DEVICE_PROP& c1, OPENCL_DEVICE_PROP& c2, bool loose) {
@@ -169,7 +173,7 @@ int compareOSVersionTo(int toMajor, int toMinor) {
         p1 = strchr(vers, '.');
         minor = atoi(p1+1);
     }
-    
+
     if (major < toMajor) return -1;
     if (major > toMajor) return 1;
     // if (major == toMajor) compare minor version numbers
@@ -179,6 +183,72 @@ int compareOSVersionTo(int toMajor, int toMinor) {
 }
 #endif
 
+#ifdef ANDROID
+#include <android/dlext.h>
+void* (*p_android_dlopen_ext)(const char*, int, const android_dlextinfo*);
+struct android_namespace_t* (*p_android_create_namespace)(const char*, const char*, const char*, uint64_t, const char*, struct android_namespace_t*);
+struct android_namespace_t* (*p_android_get_exported_namespace)(const char*);
+
+struct android_namespace_t* get_android_namespace(vector<string>& warnings) {
+    p_android_get_exported_namespace = (struct android_namespace_t*(*)(const char*)) dlsym(RTLD_DEFAULT, "android_get_exported_namespace");
+    if (!p_android_get_exported_namespace) {
+        gpu_warning(warnings, "No android_get_exported_namespace()");
+    }
+    if (!p_android_get_exported_namespace) {
+        p_android_get_exported_namespace = (struct android_namespace_t*(*)(const char*)) dlsym(RTLD_DEFAULT, "__loader_android_get_exported_namespace");
+        if (!p_android_get_exported_namespace) {
+            gpu_warning(warnings, "No __loader_android_get_exported_namespace()");
+        }
+    }
+    if (p_android_get_exported_namespace) {
+        return (*p_android_get_exported_namespace)("vndk");
+    }
+
+    p_android_create_namespace = (struct android_namespace_t*(*)(const char*, const char*, const char*, uint64_t, const char*, struct android_namespace_t*)) dlsym(RTLD_DEFAULT, "android_create_namespace");
+    if (!p_android_create_namespace) {
+        gpu_warning(warnings, "No android_create_namespace()");
+        return NULL;
+    }
+    string lib_path;
+    if (sizeof(void*) == 8) {
+        lib_path = "/system/lib64/";
+    }
+    else {
+        lib_path = "/system/lib/";
+    }
+#define ANDROID_NAMESPACE_TYPE_ISOLATED 1
+#define ANDROID_NAMESPACE_TYPE_SHARED 2
+    return (*p_android_create_namespace)("trustme", lib_path.c_str(), lib_path.c_str(), ANDROID_NAMESPACE_TYPE_SHARED | ANDROID_NAMESPACE_TYPE_ISOLATED, "/system/:/data/:/vendor/", NULL);
+}
+
+void* android_dlopen(const char* filename, vector<string>& warnings) {
+    char buf[256];
+    gpu_warning(warnings, "Trying dlopen()");
+    void* handle = dlopen(filename, RTLD_NOW);
+    if (handle) {
+        return handle;
+    }
+
+    p_android_dlopen_ext = (void*(*)(const char*, int, const android_dlextinfo*)) dlsym(RTLD_DEFAULT, "android_dlopen_ext");
+    if (!p_android_dlopen_ext) {
+        gpu_warning(warnings, "No android_dlopen_ext()");
+        return NULL;
+    }
+
+    struct android_namespace_t* ns = get_android_namespace(warnings);
+    if (!ns) {
+        gpu_warning(warnings, "No namespace");
+        return NULL;
+    }
+
+    const android_dlextinfo dlextinfo = {
+        .flags = ANDROID_DLEXT_USE_NAMESPACE,
+        .library_namespace = ns,
+    };
+    gpu_warning(warnings, "Trying android_dlopen_ext()");
+    return (*p_android_dlopen_ext)(filename, RTLD_NOW, &dlextinfo);
+}
+#endif
 
 // OpenCL interfaces are documented here:
 // http://www.khronos.org/registry/cl/sdk/1.0/docs/man/xhtml/ and
@@ -211,7 +281,7 @@ void COPROCS::get_opencl(
 #ifdef _WIN32
     opencl_lib = LoadLibrary("OpenCL.dll");
     if (!opencl_lib) {
-        warnings.push_back("No OpenCL library found");
+        gpu_warning(warnings, "No OpenCL library found");
         return;
     }
 
@@ -222,6 +292,8 @@ void COPROCS::get_opencl(
 #else
 #ifdef __APPLE__
     opencl_lib = dlopen("/System/Library/Frameworks/OpenCL.framework/Versions/Current/OpenCL", RTLD_NOW);
+#elif defined ANDROID
+    opencl_lib = android_dlopen("libOpenCL.so", warnings);
 #else
     opencl_lib = dlopen("libOpenCL.so", RTLD_NOW);
     if (!opencl_lib) {
@@ -229,8 +301,8 @@ void COPROCS::get_opencl(
     }
 #endif
     if (!opencl_lib) {
-        sprintf(buf, "OpenCL: %s", dlerror());
-        warnings.push_back(buf);
+        snprintf(buf, sizeof(buf), "OpenCL: %s", dlerror());
+        gpu_warning(warnings, buf);
         return;
     }
     p_clGetPlatformIDs = (cl_int(*)(cl_uint, cl_platform_id*, cl_uint*)) dlsym( opencl_lib, "clGetPlatformIDs" );
@@ -240,25 +312,25 @@ void COPROCS::get_opencl(
 #endif
 
     if (!p_clGetPlatformIDs) {
-        warnings.push_back("clGetPlatformIDs() missing from OpenCL library");
+        gpu_warning(warnings, "clGetPlatformIDs() missing from OpenCL library");
         goto leave;
     }
     if (!p_clGetPlatformInfo) {
-        warnings.push_back("clGetPlatformInfo() missing from OpenCL library");
+        gpu_warning(warnings, "clGetPlatformInfo() missing from OpenCL library");
         goto leave;
     }
     if (!p_clGetDeviceIDs) {
-        warnings.push_back("clGetDeviceIDs() missing from OpenCL library");
+        gpu_warning(warnings, "clGetDeviceIDs() missing from OpenCL library");
         goto leave;
     }
     if (!p_clGetDeviceInfo) {
-        warnings.push_back("clGetDeviceInfo() missing from OpenCL library");
+        gpu_warning(warnings, "clGetDeviceInfo() missing from OpenCL library");
         goto leave;
     }
 
     ciErrNum = (*p_clGetPlatformIDs)(MAX_OPENCL_PLATFORMS, platforms, &num_platforms);
     if ((ciErrNum != CL_SUCCESS) || (num_platforms == 0)) {
-        warnings.push_back("clGetPlatformIDs() failed to return any OpenCL platforms");
+        gpu_warning(warnings, "clGetPlatformIDs() failed to return any OpenCL platforms");
         goto leave;
     }
 
@@ -282,10 +354,10 @@ void COPROCS::get_opencl(
         );
         if (ciErrNum != CL_SUCCESS) {
             snprintf(buf, sizeof(buf),
-                "Couldn't get PLATFORM_VERSION for platform #%d; error %d",
+                "Couldn't get PLATFORM_VERSION for platform #%u; error %d",
                 platform_index, ciErrNum
             );
-            warnings.push_back(buf);
+            gpu_warning(warnings, buf);
             continue;
         }
 
@@ -295,10 +367,10 @@ void COPROCS::get_opencl(
         );
         if (ciErrNum != CL_SUCCESS) {
             snprintf(buf, sizeof(buf),
-                "Couldn't get PLATFORM_VENDOR for platform #%d; error %d",
+                "Couldn't get PLATFORM_VENDOR for platform #%u; error %d",
                 platform_index, ciErrNum
             );
-            warnings.push_back(buf);
+            gpu_warning(warnings, buf);
         }
 
         //////////// CPU //////////////
@@ -312,10 +384,10 @@ void COPROCS::get_opencl(
             num_devices = 0;                 // No devices
             if (ciErrNum != CL_DEVICE_NOT_FOUND) {
                 snprintf(buf, sizeof(buf),
-                    "Couldn't get CPU Device IDs for platform #%d: error %d",
+                    "Couldn't get CPU Device IDs for platform #%u: error %d",
                     platform_index, ciErrNum
                 );
-                warnings.push_back(buf);
+                gpu_warning(warnings, buf);
             }
         }
 
@@ -340,24 +412,61 @@ void COPROCS::get_opencl(
         }
 
         //////////// GPUs and Accelerators //////////////
-        
+
+// Looks like implementation of Qualcomm has some problems with clGetDeviceIDs
+// It returns CL_DEVICE_NOT_FOUND for CL_DEVICE_TYPE_GPU and CL_DEVICE_TYPE_ACCELERATOR combined
+// But it returns CL_SUCCESS when asking separately for CL_DEVICE_TYPE_GPU or CL_DEVICE_TYPE_ACCELERATOR
+// So we will ask for CL_DEVICE_TYPE_GPU and CL_DEVICE_TYPE_ACCELERATOR separately
+#ifdef ANDROID
+        cl_device_id android_gpu[MAX_COPROC_INSTANCES];
+        cl_uint num_android_gpu = 0;
+        ciErrNum = (*p_clGetDeviceIDs)(
+            platforms[platform_index],
+            (CL_DEVICE_TYPE_GPU),
+            MAX_COPROC_INSTANCES, android_gpu, &num_android_gpu
+        );
+        if (ciErrNum == CL_SUCCESS && num_android_gpu > 0) {
+            for (int i=0; i<num_android_gpu; ++i) {
+                devices[i] = android_gpu[i];
+            }
+            num_devices = num_android_gpu;
+        }
+
+        cl_device_id android_acc[MAX_COPROC_INSTANCES];
+        cl_uint num_android_acc = 0;
+        ciErrNum = (*p_clGetDeviceIDs)(
+            platforms[platform_index],
+            (CL_DEVICE_TYPE_ACCELERATOR),
+            MAX_COPROC_INSTANCES - num_devices, android_acc, &num_android_acc
+        );
+        if (ciErrNum == CL_SUCCESS && num_android_acc > 0) {
+            for (int i=0; i<num_android_acc; ++i) {
+                devices[num_devices+i] = android_acc[i];
+            }
+            num_devices += num_android_acc;
+        }
+#else
         ciErrNum = (*p_clGetDeviceIDs)(
             platforms[platform_index],
             (CL_DEVICE_TYPE_GPU | CL_DEVICE_TYPE_ACCELERATOR),
             MAX_COPROC_INSTANCES, devices, &num_devices
         );
 
-        if (ciErrNum == CL_DEVICE_NOT_FOUND) continue;  // No devices
-        if (num_devices == 0) continue;                 // No devices
+        if (ciErrNum == CL_DEVICE_NOT_FOUND) {
+            gpu_warning(warnings, "No OpenCL GPUs or Accelerators found");
+            continue;  // No devices
+        }
 
         if (ciErrNum != CL_SUCCESS) {
             snprintf(buf, sizeof(buf),
-                "Couldn't get Device IDs for platform #%d: error %d",
+                "Couldn't get Device IDs for platform #%u: error %d",
                 platform_index, ciErrNum
             );
-            warnings.push_back(buf);
+            gpu_warning(warnings, buf);
             continue;
         }
+#endif
+        if (num_devices == 0) continue;                 // No devices
 
         // Mac OpenCL does not recognize all NVIDIA GPUs returned by CUDA
         // Fortunately, CUDA and OpenCL return the same GPU model name on
@@ -391,9 +500,7 @@ void COPROCS::get_opencl(
                 }
                 if (numToMatch == (int)num_devices) break;
                 if (numToMatch < (int)num_devices) {
-                    warnings.push_back(
-                        "Could not match ATI OpenCL and CAL GPUs: ignoring CAL."
-                    );
+                    gpu_warning(warnings, "Could not match ATI OpenCL and CAL GPUs: ignoring CAL.");
                     // If we can't match ATI OpenCL and CAL GPUs, ignore CAL
                     // and keep OpenCL because AMD has deprecated CAL.
                     ati_gpus.clear();
@@ -432,27 +539,27 @@ void COPROCS::get_opencl(
             prop.is_used = COPROC_UNUSED;
             prop.get_device_version_int();
 
-            //////////// NVIDIA //////////////
             if (is_NVIDIA(prop.vendor)) {
+                //////////// NVIDIA //////////////
                 bool cuda_match_found = false;
                 if (nvidia.have_cuda) {
                     // Mac OpenCL does not recognize all NVIDIA GPUs returned by
-                    // CUDA but we assume that OpenCL and CUDA return devices 
+                    // CUDA but we assume that OpenCL and CUDA return devices
                     // with identical model name strings and that OpenCL returns
-                    // devices in order of acending PCI slot.
+                    // devices in order of ascending PCI slot.
                     //
-                    // On other systems, assume OpenCL and CUDA return devices 
+                    // On other systems, assume OpenCL and CUDA return devices
                     // in the same order.
                     //
                     int saved_CUDA_index = current_CUDA_index;
-                    
+
                     while (1) {
                         if (current_CUDA_index >= (int)(nvidia_gpus.size())) {
                             snprintf(buf, sizeof(buf),
-                                "OpenCL NVIDIA index #%d does not match any CUDA device",
+                                "OpenCL NVIDIA index #%u does not match any CUDA device",
                                 device_index
                             );
-                            warnings.push_back(buf);
+                            gpu_warning(warnings, buf);
                             // Newer versions of CUDA driver don't support older NVIDIA GPUs
                             if (nvidia.cuda_version >= 6050) {
                                 prop.device_num = (int)(nvidia_opencls.size());
@@ -488,7 +595,7 @@ void COPROCS::get_opencl(
                     c.opencl_prop = prop;
                     c.set_peak_flops();
                     if (c.bad_gpu_peak_flops("NVIDIA OpenCL", s)) {
-                        warnings.push_back(s);
+                        gpu_warning(warnings, s.c_str());
                     }
                     prop.peak_flops = c.peak_flops;
                 }
@@ -499,18 +606,16 @@ void COPROCS::get_opencl(
                 } else {
                     prop.opencl_available_ram = prop.global_mem_size;
                 }
-                
+
                 // Build nvidia_opencls vector in device_num order
                 for (it=nvidia_opencls.begin(); it != nvidia_opencls.end(); ++it) {
                     if (it->device_num > prop.device_num) break;
                 }
                 nvidia_opencls.insert(it, prop);
-                
+
                 if (cuda_match_found) ++current_CUDA_index;
-            }
-            
-            //////////// AMD / ATI //////////////
-            else if (is_AMD(prop.vendor)) {
+            } else if (is_AMD(prop.vendor)) {
+                //////////// AMD / ATI //////////////
                 prop.opencl_device_index = device_index;
 
                 if (ati.have_cal) {
@@ -522,10 +627,10 @@ void COPROCS::get_opencl(
                     while (1) {
                         if (current_CAL_index >= num_CAL_devices) {
                             snprintf(buf, sizeof(buf),
-                                "OpenCL ATI device #%d does not match any CAL device",
+                                "OpenCL ATI device #%u does not match any CAL device",
                                 device_index
                             );
-                            warnings.push_back(buf);
+                            gpu_warning(warnings, buf);
                             goto leave; // Should never happen
                         }
                         if ((int)ati_gpus[current_CAL_index].attribs.target >= min_CAL_target) {
@@ -557,7 +662,7 @@ void COPROCS::get_opencl(
                     c.opencl_prop = prop;
                     c.set_peak_flops();
                     if (c.bad_gpu_peak_flops("AMD OpenCL", s)) {
-                        warnings.push_back(s);
+                        gpu_warning(warnings, s.c_str());
                     }
                     prop.peak_flops = c.peak_flops;
                 }
@@ -568,10 +673,8 @@ void COPROCS::get_opencl(
                     prop.opencl_available_ram = prop.global_mem_size;
                 }
                 ati_opencls.push_back(prop);
-            }
-
-            //////////// INTEL GPU //////////////
-            else if (is_intel(prop.vendor)) {
+            } else if (is_intel(prop.vendor)) {
+                //////////// INTEL GPU //////////////
                 prop.device_num = (int)(intel_gpu_opencls.size());
                 prop.opencl_device_index = device_index;
 
@@ -584,7 +687,7 @@ void COPROCS::get_opencl(
 
                 c.set_peak_flops();
                 if (c.bad_gpu_peak_flops("Intel OpenCL", s)) {
-                    warnings.push_back(s);
+                    gpu_warning(warnings, s.c_str());
                 }
                 prop.peak_flops = c.peak_flops;
                 prop.opencl_available_ram = prop.global_mem_size;
@@ -593,10 +696,29 @@ void COPROCS::get_opencl(
 
                 // At present Intel GPUs only support OpenCL
                 // and do not have a native GPGPU framework,
-                // so treat each detected Intel OpenCL GPU device as
-                // a native device.
+                // so treat each detected GPU as a native device.
                 //
                 intel_gpus.push_back(c);
+#ifdef __APPLE__
+            } else if (is_apple(prop.vendor)) {
+                //////////// APPLE GPU //////////////
+                prop.device_num = (int)(apple_gpu_opencls.size());
+                prop.opencl_device_index = device_index;
+
+                COPROC_APPLE c;
+                c.opencl_prop = prop;
+                c.is_used = COPROC_UNUSED;
+                c.available_ram = prop.global_mem_size;
+
+                c.set_peak_flops();
+                if (c.bad_gpu_peak_flops("Apple OpenCL", s)) {
+                    gpu_warning(warnings, s.c_str());
+                }
+                prop.peak_flops = c.peak_flops;
+                prop.opencl_available_ram = prop.global_mem_size;
+
+                apple_gpu_opencls.push_back(prop);
+#endif
             } else {
                 //////////// OTHER GPU OR ACCELERATOR //////////////
                 // Put each coprocessor instance into a separate other_opencls element
@@ -610,7 +732,7 @@ void COPROCS::get_opencl(
                         opencl_device_index++;  // Another OpenCL device from same vendor
                     }
                 }
-                
+
                 prop.device_num = 0;    // Each vector entry has only one device
                 prop.opencl_device_index = opencl_device_index;
                 prop.opencl_available_ram = prop.global_mem_size;
@@ -625,11 +747,11 @@ void COPROCS::get_opencl(
                 }
                 if (prop.peak_flops <= 0 || prop.peak_flops > GPU_MAX_PEAK_FLOPS) {
                     char buf2[256];
-                    sprintf(buf2,
-                        "OpenCL generic: bad peak FLOPS; Max units %d, max freq %d MHz",
+                    snprintf(buf2, sizeof(buf2),
+                        "OpenCL generic: bad peak FLOPS; Max units %u, max freq %u MHz",
                         prop.max_compute_units, prop.max_clock_frequency
                     );
-                    warnings.push_back(buf2);
+                    gpu_warning(warnings, buf2);
                     prop.peak_flops = GPU_DEFAULT_PEAK_FLOPS;
                 }
 
@@ -637,15 +759,16 @@ void COPROCS::get_opencl(
             }
         }
     }
-    
-    // Neither nvidia.count, ati.count nor intel_gpu.count have been set yet, 
-    // so we can't test have_nvidia(), have_ati() or have_intel_gpu() here.
+
+    // nvidia.count etc. haven't been set yet,
+    // so we can't use have_nvidia() etc.
     //
     if ((nvidia_opencls.size() > 0) || nvidia.have_cuda) max_other_coprocs--;
     if ((ati_opencls.size() > 0) || ati.have_cal) max_other_coprocs--;
     if (intel_gpu_opencls.size() > 0) max_other_coprocs--;
+    if (apple_gpu_opencls.size() > 0) max_other_coprocs--;
     if ((int)other_opencls.size() > max_other_coprocs) {
-        warnings.push_back("Too many OpenCL device types found");
+        gpu_warning(warnings, "Too many OpenCL device types found");
     }
 
 
@@ -666,12 +789,11 @@ void COPROCS::get_opencl(
     if ((nvidia_opencls.size() == 0) &&
         (ati_opencls.size() == 0) &&
         (intel_gpu_opencls.size() == 0) &&
+        (apple_gpu_opencls.size() == 0) &&
         (cpu_opencls.size() == 0) &&
         (other_opencls.size() == 0)
     ) {
-        warnings.push_back(
-            "OpenCL library present but no OpenCL-capable devices found"
-        );
+        gpu_warning(warnings, "OpenCL library present but no OpenCL-capable devices found");
     }
 leave:
 #ifdef _WIN32
@@ -700,7 +822,7 @@ void COPROCS::correlate_opencl(
             safe_strcpy(nvidia.prop.name, nvidia.opencl_prop.name);
         }
     }
-    
+
     if (ati_opencls.size() > 0) {
         if (ati.have_cal) { // If CAL already found the "best" CAL GPU
             ati.merge_opencl(ati_opencls, ignore_gpu_instance[PROC_TYPE_AMD_GPU]);
@@ -712,12 +834,34 @@ void COPROCS::correlate_opencl(
             safe_strcpy(ati.name, ati.opencl_prop.name);
         }
     }
-    
+
     if (intel_gpu_opencls.size() > 0) {
-        intel_gpu.find_best_opencls(use_all, intel_gpu_opencls, ignore_gpu_instance[PROC_TYPE_INTEL_GPU]);
+        intel_gpu.find_best_opencls(
+            use_all, intel_gpu_opencls,
+            ignore_gpu_instance[PROC_TYPE_INTEL_GPU]
+        );
         intel_gpu.available_ram = intel_gpu.opencl_prop.global_mem_size;
         safe_strcpy(intel_gpu.name, intel_gpu.opencl_prop.name);
     }
+#ifdef __APPLE__
+    if (apple_gpu_opencls.size() > 0) {
+        if (apple_gpu.have_metal) {
+            apple_gpu.merge_opencl(
+                apple_gpu_opencls, ignore_gpu_instance[PROC_TYPE_APPLE_GPU]
+            );
+        } else {
+            apple_gpu.find_best_opencls(
+                use_all, apple_gpu_opencls,
+                ignore_gpu_instance[PROC_TYPE_APPLE_GPU]
+            );
+            safe_strcpy(apple_gpu.model, apple_gpu.opencl_prop.name);
+        }
+        apple_gpu.set_peak_flops();
+        if (!apple_gpu.available_ram) {
+            apple_gpu.available_ram = apple_gpu.opencl_prop.global_mem_size;
+        }
+    }
+#endif
 }
 
 cl_int COPROCS::get_opencl_info(
@@ -734,7 +878,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get name for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -744,7 +888,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get vendor for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -754,7 +898,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get vendor ID for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -764,7 +908,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get availability for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -780,7 +924,7 @@ cl_int COPROCS::get_opencl_info(
                 "clGetDeviceInfo failed to get half-precision floating point capabilities for device %d",
                 (int)device_index
             );
-            warnings.push_back(buf);
+            gpu_warning(warnings, buf);
             return ciErrNum;
         }
     }
@@ -794,7 +938,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get single-precision floating point capabilities for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -810,7 +954,7 @@ cl_int COPROCS::get_opencl_info(
                 "clGetDeviceInfo failed to get double-precision floating point capabilities for device %d",
                 (int)device_index
             );
-            warnings.push_back(buf);
+            gpu_warning(warnings, buf);
             return ciErrNum;
         }
     }
@@ -824,7 +968,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get little or big endian for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -837,7 +981,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get execution capabilities for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -850,7 +994,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get device extensions for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -863,7 +1007,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get global memory size for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -876,7 +1020,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get local memory size for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -889,7 +1033,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get max clock frequency for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -902,7 +1046,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get max compute units for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -912,7 +1056,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get OpenCL version supported by device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -922,7 +1066,7 @@ cl_int COPROCS::get_opencl_info(
             "clGetDeviceInfo failed to get OpenCL driver version for device %d",
             (int)device_index
         );
-        warnings.push_back(buf);
+        gpu_warning(warnings, buf);
         return ciErrNum;
     }
 
@@ -935,7 +1079,7 @@ cl_int COPROCS::get_opencl_info(
                 "clGetDeviceInfo failed to get CL_DEVICE_COMPUTE_CAPABILITY_MAJOR_NV for device %d",
                 (int)device_index
             );
-            warnings.push_back(buf);
+            gpu_warning(warnings, buf);
             return ciErrNum;
         }
 
@@ -945,7 +1089,7 @@ cl_int COPROCS::get_opencl_info(
                 "clGetDeviceInfo failed to get CL_DEVICE_COMPUTE_CAPABILITY_MINOR_NV for device %d",
                 (int)device_index
             );
-            warnings.push_back(buf);
+            gpu_warning(warnings, buf);
             return ciErrNum;
         }
 
@@ -962,17 +1106,17 @@ cl_int COPROCS::get_opencl_info(
                 "clGetDeviceInfo failed to get AMD Board Name for device %d",
                 (int)device_index
             );
-            warnings.push_back(buf);
+            gpu_warning(warnings, buf);
             return ciErrNum;
         }
-    
+
         ciErrNum = (*p_clGetDeviceInfo)(prop.device_id, CL_DEVICE_SIMD_PER_COMPUTE_UNIT_AMD, sizeof(prop.amd_simd_per_compute_unit), &prop.amd_simd_per_compute_unit, NULL);
         if (ciErrNum != CL_SUCCESS) {
             snprintf(buf, sizeof(buf),
                 "clGetDeviceInfo failed to get CL_DEVICE_SIMD_PER_COMPUTE_UNIT_AMD for device %d",
                 (int)device_index
             );
-            warnings.push_back(buf);
+            gpu_warning(warnings, buf);
             return ciErrNum;
         }
 
@@ -982,7 +1126,7 @@ cl_int COPROCS::get_opencl_info(
                 "clGetDeviceInfo failed to get CL_DEVICE_SIMD_WIDTH_AMD for device %d",
                 (int)device_index
             );
-            warnings.push_back(buf);
+            gpu_warning(warnings, buf);
             return ciErrNum;
         }
 
@@ -992,7 +1136,7 @@ cl_int COPROCS::get_opencl_info(
                 "clGetDeviceInfo failed to get CL_DEVICE_SIMD_INSTRUCTION_WIDTH_AMD for device %d",
                 (int)device_index
             );
-            warnings.push_back(buf);
+            gpu_warning(warnings, buf);
             return ciErrNum;
         }
 
@@ -1012,9 +1156,12 @@ void COPROC::merge_opencl(
 ) {
     unsigned int i, j;
 
+    // find OpenCL info for the 'best' instance,
+    // and copy it into this object
+    //
     for (i=0; i<opencls.size(); i++) {
         opencls[i].is_used = COPROC_UNUSED;
-        
+
         if (in_vector(opencls[i].device_num, ignore_dev)) {
             opencls[i].is_used = COPROC_IGNORED;
             continue;
@@ -1027,10 +1174,10 @@ void COPROC::merge_opencl(
         }
     }
 
-    opencl_device_count = 0;
-
-    // Fill in info for other GPUs which CAL or CUDA found equivalent to best
+    // Fill in OpenCL ID info for instances
+    // which CAL or CUDA found equivalent to best instance
     //
+    opencl_device_count = 0;
     for (i=0; i<(unsigned int)count; ++i) {
         for (j=0; j<opencls.size(); j++) {
             if (device_nums[i] == opencls[j].device_num) {
@@ -1052,7 +1199,7 @@ void COPROC::find_best_opencls(
 ) {
     unsigned int i;
 
-    // identify the most capable ATI, NVIDIA or Intel OpenCL GPU
+    // identify the most capable ATI, NVIDIA, Intel, or Apple OpenCL GPU
     //
     bool first = true;
     for (i=0; i<opencls.size(); i++) {
@@ -1252,7 +1399,7 @@ void COPROCS::opencl_get_ati_mem_size_from_opengl(vector<string>& warnings) {
                                             "opencl_get_ati_mem_size_from_opengl model name mismatch: %s vs %s\n",
                                             ati_opencls[ati_gpu_index].name, (char *)CFDataGetBytePtr(modelName[j])
                                         );
-                                        warnings.push_back(buf);
+                                        gpu_warning(warnings, buf);
                                     }
                                 } else {
                                     // Could not get model name from IOKit, so use renderer name
@@ -1271,7 +1418,7 @@ void COPROCS::opencl_get_ati_mem_size_from_opengl(vector<string>& warnings) {
                                             "opencl_get_ati_mem_size_from_opengl model name to renderer mismatch: %s vs %s\n",
                                             strRend, ati_opencls[ati_gpu_index].name
                                         );
-                                        warnings.push_back(buf);
+                                        gpu_warning(warnings, buf);
                                     }
                                 }
                             }   // End if (log_flags.coproc_debug) {
@@ -1281,13 +1428,13 @@ void COPROCS::opencl_get_ati_mem_size_from_opengl(vector<string>& warnings) {
 
                         CGLDestroyContext (cglContext);
                     } else {
-                        warnings.push_back(
-                            "opencl_get_ati_mem_size_from_opengl failed to create context\n"
+                        gpu_warning(warnings,
+                            "opencl_get_ati_mem_size_from_opengl failed to create context"
                         );
                     }
                 } else {
-                    warnings.push_back(
-                        "opencl_get_ati_mem_size_from_opengl failed to create PixelFormat\n"
+                    gpu_warning(warnings,
+                        "opencl_get_ati_mem_size_from_opengl failed to create PixelFormat"
                     );
                 }
             }       // End if kCGLRPAcceleratedCompute attribute
@@ -1344,40 +1491,40 @@ static io_service_t IOServicePortFromCGDisplayID(CGDirectDisplayID displayID)
 {
     io_iterator_t iter;
     io_service_t serv, servicePort = 0;
-    
+
     CFMutableDictionaryRef matching = IOServiceMatching("IODisplayConnect");
-    
+
     // releases matching for us
     kern_return_t err = IOServiceGetMatchingServices(kIOMasterPortDefault,
                                                      matching,
                                                      &iter);
     if (err)
         return 0;
-    
+
     while ((serv = IOIteratorNext(iter)) != 0)
     {
         CFDictionaryRef info;
         CFIndex vendorID, productID, serialNumber;
         CFNumberRef vendorIDRef, productIDRef, serialNumberRef;
         Boolean success;
-        
+
         info = IODisplayCreateInfoDictionary(serv,
                                              kIODisplayOnlyPreferredName);
-        
+
         vendorIDRef = (CFNumberRef)CFDictionaryGetValue(info,
                                            CFSTR(kDisplayVendorID));
         productIDRef = (CFNumberRef)CFDictionaryGetValue(info,
                                             CFSTR(kDisplayProductID));
         serialNumberRef = (CFNumberRef)CFDictionaryGetValue(info,
                                                CFSTR(kDisplaySerialNumber));
-        
+
         success = CFNumberGetValue(vendorIDRef, kCFNumberCFIndexType,
                                    &vendorID);
         success &= CFNumberGetValue(productIDRef, kCFNumberCFIndexType,
                                     &productID);
         success &= CFNumberGetValue(serialNumberRef, kCFNumberCFIndexType,
                                     &serialNumber);
-        
+
         if (!success)
         {
             CFRelease(info);
@@ -1394,14 +1541,14 @@ static io_service_t IOServicePortFromCGDisplayID(CGDirectDisplayID displayID)
             CFRelease(info);
             continue;
         }
-        
+
         // The VendorID, Product ID, and the Serial Number all Match Up!
         // Therefore we have found the appropriate display io_service
         servicePort = serv;
         CFRelease(info);
         break;
     }
-    
+
     IOObjectRelease(iter);
     return servicePort;
 }
