@@ -15,15 +15,10 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
-#if defined(_WIN32) && !defined(__STDWX_H__) && !defined(_BOINC_WIN_) && !defined(_AFX_STDAFX_H_)
+#ifdef _WIN32
 #include "boinc_win.h"
 #endif
 
-#ifdef _MSC_VER
-#define snprintf _snprintf
-#define snprintf_s _snprintf_s
-#endif
-
 #ifdef HAVE_INTRIN_H
 #include <intrin.h>
 #endif
@@ -199,6 +194,21 @@
 #ifndef PRODUCT_ENTERPRISE_S_N_EVALUATION
 #define PRODUCT_ENTERPRISE_S_N_EVALUATION           0x00000082
 #endif
+#ifndef PRODUCT_IOTENTERPRISE
+#define PRODUCT_IOTENTERPRISE                       0x000000BC
+#endif
+#ifndef PRODUCT_IOTENTERPRISES
+#define PRODUCT_IOTENTERPRISES                      0x000000BF
+#endif
+#ifndef PRODUCT_AZURESTACKHCI_SERVER_CORE
+#define PRODUCT_AZURESTACKHCI_SERVER_CORE           0x00000196
+#endif
+#ifndef PRODUCT_DATACENTER_SERVER_AZURE_EDITION
+#define PRODUCT_DATACENTER_SERVER_AZURE_EDITION     0x00000197
+#endif
+#ifndef PRODUCT_DATACENTER_SERVER_CORE_AZURE_EDITION
+#define PRODUCT_DATACENTER_SERVER_CORE_AZURE_EDITION 0x00000198
+#endif
 
 
 // new Architecture(s)
@@ -218,7 +228,7 @@ static unsigned long long _xgetbv(unsigned int index){
       unsigned int A=0, D=0;
 
 #ifdef __GNUC__
-  #ifdef ASM_SUPPORTS_XGETBV  
+  #ifdef ASM_SUPPORTS_XGETBV
       __asm__ __volatile__("xgetbv" : "=a"(A), "=d"(D) : "c"(index));
   #else
       __asm__ __volatile__(".byte 0x0f, 0x01, 0xd0": "=a"(A), "=d"(D) : "c"(index));
@@ -253,14 +263,14 @@ static unsigned long long _xgetbv(unsigned int index){
 #else
 static void __cpuid(unsigned int cpuinfo[4], unsigned int type)  {
 #ifdef __GNUC__
-  #ifdef ASM_SUPPORTS_CPUID  
-      __asm__ __volatile__("cpuid" 
-                            : "=a" (cpuinfo[0]), "=b" (cpuinfo[1]), 
-                              "=c" (cpuinfo[2]), "=d" (cpuinfo[3]) 
+  #ifdef ASM_SUPPORTS_CPUID
+      __asm__ __volatile__("cpuid"
+                            : "=a" (cpuinfo[0]), "=b" (cpuinfo[1]),
+                              "=c" (cpuinfo[2]), "=d" (cpuinfo[3])
                             : "a" (type));
   #else
       __asm__ __volatile__(".byte 0x0f, 0xa2"
-                            : "=a" (cpuinfo[0]), "=b" (cpuinfo[1]), 
+                            : "=a" (cpuinfo[0]), "=b" (cpuinfo[1]),
                               "=c" (cpuinfo[2]), "=d" (cpuinfo[3])
                             : "a" (type));
   #endif
@@ -364,11 +374,23 @@ int get_os_information(
     switch (osvi.dwPlatformId) {
         case VER_PLATFORM_WIN32_NT:
 
-            if ( osvi.dwMajorVersion == 10 && osvi.dwMinorVersion == 0 ) {
+            if ( osvi.dwMajorVersion == 10 && osvi.dwMinorVersion == 0) {
                 if( osvi.wProductType == VER_NT_WORKSTATION ) {
-                    strlcat(os_name, "Windows 10", os_name_size);
+                    if ( osvi.dwBuildNumber >= 22000 ) {
+                        strlcat(os_name, "Windows 11", os_name_size);
+                    } else {
+                        strlcat(os_name, "Windows 10", os_name_size);
+                    }
                 } else {
-                    strlcat(os_name, "Windows Server 2016", os_name_size);
+                    if (osvi.dwBuildNumber >= 25398) {
+                        strlcat(os_name, "Windows Server 23H2", os_name_size);
+                    } else if (osvi.dwBuildNumber >= 20348) {
+                        strlcat(os_name, "Windows Server 2022", os_name_size);
+                    } else if ( osvi.dwBuildNumber >= 17623) {
+                        strlcat(os_name, "Windows Server 2019", os_name_size);
+                    } else {
+                        strlcat(os_name, "Windows Server 2016", os_name_size);
+                    }
                 }
                 pGPI( 10, 0, 0, 0, &dwType);
             }
@@ -467,7 +489,7 @@ int get_os_information(
     }
 
 
-    snprintf_s( szVersion, sizeof(szVersion), ", (%.2u.%.2u.%.4u.%.2u)",
+    snprintf( szVersion, sizeof(szVersion), ", (%.2u.%.2u.%.4u.%.2u)",
         osvi.dwMajorVersion, osvi.dwMinorVersion, (osvi.dwBuildNumber & 0xFFFF), 0
     );
 
@@ -524,6 +546,18 @@ int get_os_information(
                             case PRODUCT_ENTERPRISE_N_EVALUATION:
                                 safe_strcat(szSKU, "Enterprise N (Evaluation) ");
                                 break;
+                            case PRODUCT_ENTERPRISE_S:
+                                safe_strcat(szSKU, "Enterprise LTSB ");
+                                break;
+                            case PRODUCT_ENTERPRISE_S_EVALUATION:
+                                safe_strcat(szSKU, "Enterprise LTSB Evaluation ");
+                                break;
+                            case PRODUCT_ENTERPRISE_S_N:
+                                safe_strcat(szSKU, "Enterprise LTSB N ");
+                                break;
+                            case PRODUCT_ENTERPRISE_S_N_EVALUATION:
+                                safe_strcat(szSKU, "Enterprise LTSB N Evaluation ");
+                                break;
                             case PRODUCT_HOME_BASIC:
                                 safe_strcat(szSKU, "Home Basic ");
                                 break;
@@ -542,6 +576,12 @@ int get_os_information(
                             case PRODUCT_HOME_PREMIUM_N:
                                 safe_strcat(szSKU, "Home Premium N ");
                                 break;
+                            case PRODUCT_IOTENTERPRISE:
+                                safe_strcat(szSKU, "IoT Enterprise ");
+                                break;
+                            case PRODUCT_IOTENTERPRISES:
+                                safe_strcat(szSKU, "IoT Enterprise LTSC ");
+                                break;
                             case PRODUCT_IOTUAP:
                                 safe_strcat(szSKU, "Internet of Things ");
                                 break;
@@ -590,6 +630,12 @@ int get_os_information(
                             case PRODUCT_ULTIMATE_N:
                                 safe_strcat(szSKU, "Ultimate N ");
                                 break;
+                            case PRODUCT_PRO_WORKSTATION:
+                                safe_strcat(szSKU, "Pro for Workstations ");
+                                break;
+                            case PRODUCT_PRO_WORKSTATION_N:
+                                safe_strcat(szSKU, "Pro for Workstations N ");
+                                break;
                         }
 
                     } else if( (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2) ) {
@@ -634,6 +680,9 @@ int get_os_information(
                             case PRODUCT_ARM64_SERVER:
                                 safe_strcat(szSKU, "ARM64 Server ");
                                 break;
+                            case PRODUCT_AZURESTACKHCI_SERVER_CORE:
+                                safe_strcat(szSKU, "Azure Stack HCI ");
+                                break;
                             case PRODUCT_CLOUD_HOST_INFRASTRUCTURE_SERVER:
                                 safe_strcat(szSKU, "Cloud Host Infrastructure Server ");
                                 break;
@@ -649,12 +698,24 @@ int get_os_information(
                             case PRODUCT_DATACENTER_EVALUATION_SERVER:
                                 safe_strcat(szSKU, "Datacenter (Evaluation) ");
                                 break;
+                            case PRODUCT_DATACENTER_A_SERVER_CORE:
+                                safe_strcat(szSKU, "Datacenter, Semi-Annual Channel (core installation) ");
+                                break;
+                            case PRODUCT_STANDARD_A_SERVER_CORE:
+                                safe_strcat(szSKU, "Standard, Semi-Annual Channel (core installation) ");
+                                break;
                             case PRODUCT_DATACENTER_SERVER:
                                 safe_strcat(szSKU, "Datacenter ");
                                 break;
+                            case PRODUCT_DATACENTER_SERVER_AZURE_EDITION:
+                                safe_strcat(szSKU, "Datacenter Azure ");
+                                break;
                             case PRODUCT_DATACENTER_SERVER_CORE:
                                 safe_strcat(szSKU, "Datacenter (core installation) ");
                                 break;
+                            case PRODUCT_DATACENTER_SERVER_CORE_AZURE_EDITION:
+                                safe_strcat(szSKU, "Datacenter Azure (core installation) ");
+                                break;
                             case PRODUCT_DATACENTER_SERVER_CORE_V:
                                 safe_strcat(szSKU, "Datacenter (core installation without Hyper-V) ");
                                 break;
@@ -793,6 +854,24 @@ int get_os_information(
                             case PRODUCT_WEB_SERVER_CORE:
                                 safe_strcat(szSKU, "Web Server (core installation) ");
                                 break;
+                            case PRODUCT_SB_SOLUTION_SERVER:
+                                safe_strcat(szSKU, "Small Business Server Essentials ");
+                                break;
+                            case PRODUCT_SB_SOLUTION_SERVER_EM:
+                                safe_strcat(szSKU, "Server For SB Solutions EM ");
+                                break;
+                            case PRODUCT_SERVER_FOR_SB_SOLUTIONS:
+                                safe_strcat(szSKU, "Server For SB Solutions ");
+                                break;
+                            case PRODUCT_SERVER_FOR_SB_SOLUTIONS_EM:
+                                safe_strcat(szSKU, "Server For SB Solutions EM ");
+                                break;
+                            case PRODUCT_STANDARD_SERVER_SOLUTIONS:
+                                safe_strcat(szSKU, "Server Solutions Premium ");
+                                break;
+                            case PRODUCT_STANDARD_SERVER_SOLUTIONS_CORE:
+                                safe_strcat(szSKU, "Server Solutions Premium (core installation) ");
+                                break;
                         }
 
                     } else if( (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion == 2) ) {
@@ -951,6 +1030,7 @@ int get_os_information(
 // Handle the cpuid instruction on supported compilers
 // NOTE: This only handles structured exceptions with Microsoft compilers.
 //
+#if !defined(_M_ARM) && !defined(_M_ARM64)
 int get_cpuid(unsigned int info_type, unsigned int& a, unsigned int& b, unsigned int& c, unsigned int& d) {
 
 
@@ -973,19 +1053,35 @@ int get_cpuid(unsigned int info_type, unsigned int& a, unsigned int& b, unsigned
 #endif
     return retval;
 }
+#endif
 
+void get_processor_info_from_registry(const char* name, char* info, int info_size) {
+    char inBuffer[BUFSIZ] = "";
+    HKEY hKey;
+    DWORD gotType, gotSize = BUFSIZ;
+    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", 0, KEY_READ, &hKey) == ERROR_SUCCESS) {
+        if (!RegQueryValueExA(hKey, name, 0, &gotType, (PBYTE)(inBuffer), &gotSize)) {
+            if ((gotType == REG_SZ) && strlen(inBuffer)) {
+                strlcpy(info, inBuffer, info_size);
+            }
+        }
+        RegCloseKey(hKey);
+    }
+}
 
 // Returns the processor vendor.
 // see: http://msdn.microsoft.com/en-us/library/hskdteyh.aspx
 // see: http://www.intel.com/Assets/PDF/appnote/241618.pdf
 // see: http://www.amd.com/us-en/assets/content_type/white_papers_and_tech_docs/25481.pdf
 int get_processor_vendor(char* name, int name_size) {
-    unsigned int eax = 0, ebx = 0, ecx = 0, edx = 0;
-
     if (!name) return ERR_INVALID_PARAM;
     if (name_size < 13) return ERR_WRONG_SIZE;
 
     memset(name, 0, sizeof(name_size));
+#if defined(_M_ARM) || defined(_M_ARM64)
+    get_processor_info_from_registry("VendorIdentifier", name, name_size);
+#else
+    unsigned int eax = 0, ebx = 0, ecx = 0, edx = 0;
 
     if (!get_cpuid(0x00000000, eax, ebx, ecx, edx)) {
         *((int*)(name + 0)) = ebx;
@@ -993,10 +1089,23 @@ int get_processor_vendor(char* name, int name_size) {
         *((int*)(name + 8)) = ecx;
         *((int*)(name + 12)) = '\0';
     }
+#endif
     return 0;
 }
 
+#if defined(_M_ARM) || defined(_M_ARM64)
+int get_processor_version(char* version, int version_size)
+{
+    if (!version) return ERR_INVALID_PARAM;
+    if (version_size < 128) return ERR_WRONG_SIZE;
 
+    memset(version, 0, sizeof(version_size));
+
+    get_processor_info_from_registry("Identifier", version, version_size);
+
+    return 0;
+}
+#else
 // Returns the processor family, model, stepping.
 // see: http://msdn.microsoft.com/en-us/library/hskdteyh.aspx
 // see: http://www.intel.com/Assets/PDF/appnote/241618.pdf
@@ -1014,6 +1123,7 @@ int get_processor_version(int& family, int& model, int& stepping) {
     }
     return 0;
 }
+#endif
 
 
 // Returns the processor name.
@@ -1021,14 +1131,17 @@ int get_processor_version(int& family, int& model, int& stepping) {
 // see: http://www.intel.com/Assets/PDF/appnote/241618.pdf
 // see: http://www.amd.com/us-en/assets/content_type/white_papers_and_tech_docs/25481.pdf
 int get_processor_name(char* name, int name_size) {
-    unsigned int eax = 0, ebx = 0, ecx = 0, edx = 0;
-    size_t i = 0;
-
     if (!name) return ERR_INVALID_PARAM;
     if (name_size < 48) return ERR_WRONG_SIZE;
 
     memset(name, 0, sizeof(name_size));
 
+#if defined(_M_ARM) || defined(_M_ARM64)
+    get_processor_info_from_registry("ProcessorNameString", name, name_size);
+#else
+    unsigned int eax = 0, ebx = 0, ecx = 0, edx = 0;
+    size_t i = 0;
+
     get_cpuid(0x80000000, eax, ebx, ecx, edx);
     if (!(eax >= 0x80000004)) return ERR_NOT_IMPLEMENTED;
 
@@ -1059,7 +1172,7 @@ int get_processor_name(char* name, int name_size) {
             *(name + i) = ' ';
         }
     }
-
+#endif
     return 0;
 }
 
@@ -1070,6 +1183,21 @@ int get_processor_name(char* name, int name_size) {
 // see: http://www.amd.com/us-en/assets/content_type/white_papers_and_tech_docs/25481.pdf
 //
 int get_processor_cache(int& cache) {
+#if defined(_M_ARM) || defined(_M_ARM64)
+    DWORD buffer_size = 0;
+    GetLogicalProcessorInformation(0, &buffer_size);
+    SYSTEM_LOGICAL_PROCESSOR_INFORMATION* buffer = (SYSTEM_LOGICAL_PROCESSOR_INFORMATION*)malloc(buffer_size);
+    GetLogicalProcessorInformation(&buffer[0], &buffer_size);
+
+    for (DWORD i = 0; i != buffer_size / sizeof(SYSTEM_LOGICAL_PROCESSOR_INFORMATION); ++i) {
+        if (buffer[i].Relationship == RelationCache && buffer[i].Cache.Level == 1) {
+            cache = buffer[i].Cache.LineSize;
+            break;
+        }
+    }
+
+    free(buffer);
+#else
     unsigned int eax = 0, ebx = 0, ecx = 0, edx = 0;
 
     get_cpuid(0x80000000, eax, ebx, ecx, edx);
@@ -1078,6 +1206,7 @@ int get_processor_cache(int& cache) {
     if (!get_cpuid(0x80000006, eax, ebx, ecx, edx)) {
         cache = ((ecx >> 16) & 0xffff) * 1024;
     }
+#endif
     return 0;
 }
 
@@ -1091,7 +1220,8 @@ int get_processor_cache(int& cache) {
 bool is_avx_supported() {
 
     bool supported = false;
- 
+
+#if !defined(_M_ARM) && !defined(_M_ARM64)
     // Checking for AVX on Windows requires 3 things:
     // 1) CPUID indicates that the OS uses XSAVE and XRSTORE
     //     instructions (allowing saving YMM registers on context
@@ -1104,17 +1234,18 @@ bool is_avx_supported() {
     // the instruction needs to be conditionally run.
     unsigned int a,b,c,d;
     get_cpuid(1, a, b, c, d);
- 
+
     bool osUsesXSAVE_XRSTORE = c & (1 << 27) || false;
     bool cpuAVXSuport = c & (1 << 28) || false;
- 
+
     if (osUsesXSAVE_XRSTORE && cpuAVXSuport)
     {
         // Check if the OS will save the YMM registers
         unsigned long long xcrFeatureMask = _xgetbv(_XCR_XFEATURE_ENABLED_MASK);
         supported = (xcrFeatureMask & 0x6) || false;
     }
- 
+#endif
+
     return supported;
 }
 
@@ -1127,17 +1258,29 @@ bool is_avx_supported() {
     if (feature_set_supported && test) strlcat(features, feature_name, features_size)
 
 int get_processor_features(char* vendor, char* features, int features_size) {
-    unsigned int std_eax = 0, std_ebx = 0, std_ecx = 0, std_edx = 0;
-    unsigned int ext_eax = 0, ext_ebx = 0, ext_ecx = 0, ext_edx = 0;
-	unsigned int struc_eax = 0, struc_ebx = 0, struc_ecx = 0, struc_edx = 0;
-    unsigned int std_supported = 0, ext_supported = 0, struc_ext_supported = 0, intel_supported = 0, amd_supported = 0, hygon_supported = 0;
-
     if (!vendor) return ERR_INVALID_PARAM;
     if (!features) return ERR_INVALID_PARAM;
     if (features_size < 250) return ERR_WRONG_SIZE;
 
     memset(features, 0, sizeof(features_size));
 
+#if defined(_M_ARM) || defined(_M_ARM64)
+    FEATURE_TEST(1, IsProcessorFeaturePresent(PF_ARM_VFP_32_REGISTERS_AVAILABLE), "vfp32 ");
+    FEATURE_TEST(1, IsProcessorFeaturePresent(PF_ARM_NEON_INSTRUCTIONS_AVAILABLE), "neon ");
+    FEATURE_TEST(1, IsProcessorFeaturePresent(PF_ARM_DIVIDE_INSTRUCTION_AVAILABLE), "divide ");
+    FEATURE_TEST(1, IsProcessorFeaturePresent(PF_ARM_64BIT_LOADSTORE_ATOMIC), "64lsatomic ");
+    FEATURE_TEST(1, IsProcessorFeaturePresent(PF_ARM_EXTERNAL_CACHE_AVAILABLE), "excache ");
+    FEATURE_TEST(1, IsProcessorFeaturePresent(PF_ARM_FMAC_INSTRUCTIONS_AVAILABLE), "fmac ");
+    FEATURE_TEST(1, IsProcessorFeaturePresent(PF_ARM_V8_INSTRUCTIONS_AVAILABLE), "v8 ");
+    FEATURE_TEST(1, IsProcessorFeaturePresent(PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE), "crypto ");
+    FEATURE_TEST(1, IsProcessorFeaturePresent(PF_ARM_V8_CRC32_INSTRUCTIONS_AVAILABLE), "crc32 ");
+    FEATURE_TEST(1, IsProcessorFeaturePresent(PF_ARM_V81_ATOMIC_INSTRUCTIONS_AVAILABLE), "atomic ");
+#else
+    unsigned int std_eax = 0, std_ebx = 0, std_ecx = 0, std_edx = 0;
+    unsigned int ext_eax = 0, ext_ebx = 0, ext_ecx = 0, ext_edx = 0;
+	unsigned int struc_eax = 0, struc_ebx = 0, struc_ecx = 0, struc_edx = 0;
+    unsigned int std_supported = 0, ext_supported = 0, struc_ext_supported = 0, intel_supported = 0, amd_supported = 0, hygon_supported = 0;
+
     if (strcmp(vendor, "GenuineIntel") == 0) {
         intel_supported = 1;
     }
@@ -1165,7 +1308,7 @@ int get_processor_features(char* vendor, char* features, int features_size) {
 		struc_ext_supported = 1;
 		get_cpuid(0x00000007, struc_eax, struc_ebx, struc_ecx, struc_edx);
 	}
-    
+
     FEATURE_TEST(std_supported, (std_edx & (1 << 0)), "fpu ");
     FEATURE_TEST(std_supported, (std_edx & (1 << 1)), "vme ");
     FEATURE_TEST(std_supported, (std_edx & (1 << 2)), "de ");
@@ -1196,6 +1339,7 @@ int get_processor_features(char* vendor, char* features, int features_size) {
     FEATURE_TEST(std_supported, (std_edx & (1 << 29)), "tm ");
 
     FEATURE_TEST(std_supported, (std_ecx & (1 << 0)), "pni ");
+    FEATURE_TEST(std_supported, (std_ecx & (1 << 1)), "pclmulqdq ");
     FEATURE_TEST(std_supported, (std_ecx & (1 << 9)), "ssse3 ");
 	FEATURE_TEST(std_supported, (std_ecx & (1 << 12)), "fma ");
 	FEATURE_TEST(std_supported, (std_ecx & (1 << 13)), "cx16 ");
@@ -1217,6 +1361,24 @@ int get_processor_features(char* vendor, char* features, int features_size) {
 
     if (is_avx_supported() && struc_ext_supported) {
 		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 5)), "avx2 ");
+		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 16)), "avx512f ");
+		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 17)), "avx512dq ");
+		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 19)), "adx ");
+		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 21)), "avx512ifma ");
+		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 26)), "avx512pf ");
+		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 27)), "avx512er ");
+		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 28)), "avx512cd ");
+		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 30)), "avx512bw ");
+		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 31)), "avx512vl ");
+
+		FEATURE_TEST(struc_ext_supported, (struc_ecx & (1 << 1)), "avx512vbmi ");
+		FEATURE_TEST(struc_ext_supported, (struc_ecx & (1 << 6)), "avx512_vbmi2 ");
+		FEATURE_TEST(struc_ext_supported, (struc_ecx & (1 << 8)), "gfni ");
+		FEATURE_TEST(struc_ext_supported, (struc_ecx & (1 << 9)), "vaes ");
+		FEATURE_TEST(struc_ext_supported, (struc_ecx & (1 << 10)), "vpclmulqdq ");
+		FEATURE_TEST(struc_ext_supported, (struc_ecx & (1 << 11)), "avx512_vnni ");
+		FEATURE_TEST(struc_ext_supported, (struc_ecx & (1 << 12)), "avx512_bitalg ");
+		FEATURE_TEST(struc_ext_supported, (struc_ecx & (1 << 14)), "avx512_vpopcntdq ");
     }
 
     if (intel_supported) {
@@ -1260,6 +1422,7 @@ int get_processor_features(char* vendor, char* features, int features_size) {
 		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 7)), "smep ");
 		FEATURE_TEST(struc_ext_supported, (struc_ebx & (1 << 8)), "bmi2 ");
 	}
+#endif
 
     strip_whitespace(features);
     return 0;
@@ -1298,26 +1461,37 @@ int get_processor_info(
     char* p_vendor, int p_vendor_size, char* p_model, int p_model_size,
     char* p_features, int p_features_size, double& p_cache, int& p_ncpus
 ) {
-    int family = 0, model = 0, stepping = 0, cache = 0;
-    char vendor_name[256], processor_name[256], features[P_FEATURES_SIZE];
-
+    char vendor_name[256];
     get_processor_vendor(vendor_name, sizeof(vendor_name));
-    get_processor_version(family, model, stepping);
-    get_processor_name(processor_name, sizeof(processor_name));
-    get_processor_cache(cache);
-    get_processor_features(vendor_name, features, sizeof(features));
-    get_processor_count(p_ncpus);
-
     snprintf(p_vendor, p_vendor_size, "%s", vendor_name);
 
+    char processor_name[256];
+    get_processor_name(processor_name, sizeof(processor_name));
+#if defined(_M_ARM) || defined(_M_ARM64)
+    char processor_version[256];
+    get_processor_version(processor_version, sizeof(processor_version));
+    snprintf(p_model, p_model_size,
+        "%s [%s]",
+        processor_name, processor_version
+    );
+#else
+    int family = 0, model = 0, stepping = 0;
+    get_processor_version(family, model, stepping);
     snprintf(p_model, p_model_size,
         "%s [Family %d Model %d Stepping %d]",
         processor_name, family, model, stepping
     );
+#endif
 
+    int cache = 0;
+    get_processor_cache(cache);
+    p_cache = (double)cache;
+
+    char features[P_FEATURES_SIZE];
+    get_processor_features(vendor_name, features, sizeof(features));
     snprintf(p_features, p_features_size, "%s", features);
 
-    p_cache = (double)cache;
+    get_processor_count(p_ncpus);
 
     return 0;
 }
@@ -1362,10 +1536,8 @@ int get_network_usage_totals(unsigned int& total_received, unsigned int& total_s
         }
     }
 
-    if (pIfTable != NULL) {
-        free(pIfTable);
-        pIfTable = NULL;
-    }
+    free(pIfTable);
+    pIfTable = NULL;
 
     return iRetVal;
 }
@@ -1546,7 +1718,7 @@ int HOST_INFO::get_host_battery_state() {
     SYSTEM_POWER_STATUS Status;
     ZeroMemory(&Status, sizeof(SYSTEM_POWER_STATUS));
     if (!GetSystemPowerStatus(&Status)) {
-        return false;
+        return BATTERY_STATE_UNKNOWN;
     }
 
     // Sometimes the system reports the ACLineStatus as an
