@@ -1,6 +1,6 @@
 // This file is part of BOINC.
-// http://boinc.berkeley.edu
-// Copyright (C) 2008 University of California
+// https://boinc.berkeley.edu
+// Copyright (C) 2024 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -15,6 +15,10 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with BOINC.  If not, see <http://www.gnu.org/licenses/>.
 
+// Functions for doing scheduler RPCs,
+// including creating requests and parsing replies.
+// The reply handler is in cs_scheduler.cpp
+
 #include "cpp.h"
 
 #ifdef _WIN32
@@ -27,10 +31,6 @@
 #include <ctime>
 #endif
 
-#ifdef _MSC_VER
-#define snprintf _snprintf
-#endif
-
 #include "error_numbers.h"
 #include "filesys.h"
 #include "parse.h"
@@ -73,7 +73,9 @@ bool SCHEDULER_OP::check_master_fetch_start() {
     retval = init_master_fetch(p);
     if (retval) {
         msg_printf(p, MSG_INTERNAL_ERROR,
-            "Couldn't start download of scheduler list: %s", boincerror(retval)
+            "Couldn't start download of scheduler list from %s: %s",
+            http_op.m_url,
+            boincerror(retval)
         );
         p->master_fetch_failures++;
         project_rpc_backoff(p, "scheduler list fetch failed\n");
@@ -460,13 +462,15 @@ bool SCHEDULER_OP::poll() {
                         cur_proj->nrpc_failures = 0;
                     }
                 }
-            } else {
+            }
+            else {
                 // master file fetch failed.
                 //
-                char buf[256];
-                snprintf(buf, sizeof(buf), "Scheduler list fetch failed: %s",
-                    boincerror(http_op.http_op_retval)
-                );
+                if (log_flags.sched_ops) {
+                    msg_printf(cur_proj, MSG_INFO, "Scheduler list fetch from %s failed: %s",
+                        http_op.m_url, boincerror(http_op.http_op_retval)
+                    );
+                }
                 cur_proj->master_fetch_failures++;
                 rpc_failed("Master file request failed");
             }
@@ -486,7 +490,7 @@ bool SCHEDULER_OP::poll() {
             if (http_op.http_op_retval) {
                 if (log_flags.sched_ops) {
                     msg_printf(cur_proj, MSG_INFO,
-                        "Scheduler request failed: %s", http_op.error_msg
+                        "Scheduler request to %s failed: %s", http_op.m_url, http_op.error_msg
                     );
                 }
 
@@ -590,8 +594,6 @@ int SCHEDULER_REPLY::parse(FILE* in, PROJECT* project) {
     MIOFILE mf;
     XML_PARSER xp(&mf);
     string delete_file_name;
-    bool verify_files_on_app_start = false;
-    bool non_cpu_intensive = false;
     bool ended = false;
 
     mf.init_file(in);
@@ -653,8 +655,6 @@ int SCHEDULER_REPLY::parse(FILE* in, PROJECT* project) {
             // boolean project attributes.
             // If the scheduler reply didn't specify them, they're not set.
             //
-            project->verify_files_on_app_start = verify_files_on_app_start;
-            project->non_cpu_intensive = non_cpu_intensive;
             project->ended = ended;
             return 0;
         }
@@ -859,8 +859,6 @@ int SCHEDULER_REPLY::parse(FILE* in, PROJECT* project) {
                 );
             }
             continue;
-        } else if (xp.parse_bool("non_cpu_intensive", non_cpu_intensive)) {
-            continue;
         } else if (xp.parse_bool("ended", ended)) {
             continue;
         } else if (xp.parse_bool("no_cpu_apps", btemp)) {
@@ -886,12 +884,12 @@ int SCHEDULER_REPLY::parse(FILE* in, PROJECT* project) {
                 handle_no_rsc_apps(buf, project, true);
             }
             continue;
-        } else if (xp.parse_bool("verify_files_on_app_start", verify_files_on_app_start)) {
-            continue;
         } else if (xp.parse_bool("send_full_workload", send_full_workload)) {
             continue;
         } else if (xp.parse_bool("dont_use_dcf", dont_use_dcf)) {
             continue;
+        } else if (xp.parse_bool("non_cpu_intensive", project->non_cpu_intensive)) {
+            continue;
         } else if (xp.parse_int("send_time_stats_log", send_time_stats_log)){
             continue;
         } else if (xp.parse_int("send_job_log", send_job_log)) {
@@ -937,8 +935,6 @@ int SCHEDULER_REPLY::parse(FILE* in, PROJECT* project) {
 }
 #endif
 
-USER_MESSAGE::USER_MESSAGE(char* m, char* p) {
-    message = m;
-    priority = p;
+USER_MESSAGE::USER_MESSAGE(char* m, char* p) : message(m), priority(p) {
 }
 
