@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2018 University of California
+// Copyright (C) 2023 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -21,6 +21,7 @@
 
 #ifdef __WXMAC__
 #include <Carbon/Carbon.h>
+#include <wx/uilocale.h>
 #include "filesys.h"
 #include "util.h"
 #include "mac_util.h"
@@ -35,6 +36,7 @@
 #include "miofile.h"
 #include "parse.h"
 #include "idlemon.h"
+
 #include "Events.h"
 #include "LogBOINC.h"
 #include "BOINCGUIApp.h"
@@ -74,6 +76,12 @@ bool CBOINCGUIApp::OnInit() {
     g_use_sandbox = false;
 #endif
 
+    m_isDarkMode = false;
+#if SUPPORTDARKMODE
+    wxSystemAppearance appearance = wxSystemSettings::GetAppearance();
+    m_isDarkMode = appearance.IsDark();
+#endif
+
     s_bSkipExitConfirmation = false;
     m_bFilterEvents = false;
     m_bAboutDialogIsOpen = false;
@@ -101,6 +109,7 @@ bool CBOINCGUIApp::OnInit() {
     m_iRPCPortArg = GUI_RPC_PORT;
     m_strBOINCArguments = wxEmptyString;
     m_strISOLanguageCode = wxEmptyString;
+    m_bUseDefaultLocale = true;
     m_bGUIVisible = true;
     m_bDebugSkins = false;
     m_bMultipleInstancesOK = false;
@@ -110,17 +119,20 @@ bool CBOINCGUIApp::OnInit() {
     m_iShutdownCoreClient = 0;
     m_iDisplayExitDialog = 1;
     m_iDisplayShutdownConnectedClientDialog = 1;
+#ifdef __WXMAC__
+    m_iHideMenuBarIcon = 0;
+#endif
     m_iGUISelected = BOINC_SIMPLEGUI;
     m_bSafeMessageBoxDisplayed = 0;
     m_bRunDaemon = true;
     m_bNeedRunDaemon = true;
 
     // Initialize local variables
-    int      iErrorCode = 0;
-    int      iDesiredLanguageCode = 0;
+    int      iDesiredLanguageCode = wxLANGUAGE_DEFAULT;
     bool     bOpenEventLog = false;
     wxString strDesiredSkinName = wxEmptyString;
 #ifdef SANDBOX
+    int      iErrorCode = 0;
     wxString strDialogMessage = wxEmptyString;
 #endif
     bool     success = false;
@@ -142,12 +154,18 @@ bool CBOINCGUIApp::OnInit() {
     AEInstallEventHandler( kCoreEventClass, kAEQuitApplication, NewAEEventHandlerUPP((AEEventHandlerProcPtr)QuitAppleEventHandler), 0, false );
 #endif
 
+    // Commandline parsing is done in wxApp::OnInit()
+    if (!wxApp::OnInit()) {
+        return false;
+    }
+
 #ifdef __WXMAC__
     // Don't open main window if we were started automatically at login
-    // We are launched hidden if started from our login item (except if
-    // we had windows open at logout, the system "restores" them.)
-    m_bGUIVisible = IsApplicationVisible();
-
+    if (compareOSVersionTo(13, 0) >= 0) {
+        m_bGUIVisible = !m_bBOINCMGRAutoStarted;
+    } else {
+        m_bGUIVisible = IsApplicationVisible();
+    }
     if (getTimeSinceBoot() < 30.) {
         // If the system was just started, we usually get a "Connection
         // failed" error if we try to connect too soon, so delay a bit.
@@ -156,11 +174,6 @@ bool CBOINCGUIApp::OnInit() {
 #endif
 
 
-    // Commandline parsing is done in wxApp::OnInit()
-    if (!wxApp::OnInit()) {
-        return false;
-    }
-
     if (g_use_sandbox) {
         wxCHANGE_UMASK(2);  // Set file creation mask to be writable by both user and group
                             // Our umask will be inherited by all our child processes
@@ -188,15 +201,37 @@ bool CBOINCGUIApp::OnInit() {
     m_pConfig->Read(wxT("AutomaticallyShutdownClient"), &m_iShutdownCoreClient, 0L);
     m_pConfig->Read(wxT("DisplayShutdownClientDialog"), &m_iDisplayExitDialog, 1L);
     m_pConfig->Read(wxT("DisplayShutdownConnectedClientDialog"), &m_iDisplayShutdownConnectedClientDialog, 1L);
+#ifdef __WXMAC__
+    m_pConfig->Read(wxT("HideMenuBarIcon"), &m_iHideMenuBarIcon, 0L);
+#endif
     m_pConfig->Read(wxT("DisableAutoStart"), &m_iBOINCMGRDisableAutoStart, 0L);
     m_pConfig->Read(wxT("LanguageISO"), &m_strISOLanguageCode, wxT(""));
+    m_bUseDefaultLocale = false;
+    bool bUseDefaultLocaleDefault = false;
+#ifdef __WXMAC__
+    // Because our translations don't use Apple's standard localization
+    // scheme, the Cocoa APIs used by wxLocale for wxLANGUAGE_DEFAULT
+    // always return English as the language for the reasons explained
+    // in https://stackoverflow.com/questions/48136456. The wxLocale
+    // documentation warns us to use wxUILocale::GetSystemLanguage().
+    int systemLanguageCode = wxUILocale::GetSystemLanguage();
+    wxString defaultLanguageCode = wxLocale::GetLanguageCanonicalName(systemLanguageCode);
+    bUseDefaultLocaleDefault = m_strISOLanguageCode == defaultLanguageCode;
+#else
+    const wxLanguageInfo *defaultLanguageInfo = wxLocale::GetLanguageInfo(wxLANGUAGE_DEFAULT);
+    if (defaultLanguageInfo != NULL) {
+        // Migration: assume a selected language code that matches the system default means "auto select"
+        bUseDefaultLocaleDefault = m_strISOLanguageCode == defaultLanguageInfo->CanonicalName;;
+    }
+#endif
+    m_pConfig->Read(wxT("UseDefaultLocale"), &m_bUseDefaultLocale, bUseDefaultLocaleDefault);
     m_pConfig->Read(wxT("GUISelection"), &m_iGUISelected, BOINC_SIMPLEGUI);
     m_pConfig->Read(wxT("EventLogOpen"), &bOpenEventLog);
     m_pConfig->Read(wxT("RunDaemon"), &m_bRunDaemon, 1L);
 
     // Detect if the daemon should be launched
     m_bNeedRunDaemon = m_bNeedRunDaemon && m_bRunDaemon;
-    
+
     // Should we abort the BOINC Manager startup process?
     if (m_bBOINCMGRAutoStarted && m_iBOINCMGRDisableAutoStart) {
         return false;
@@ -224,7 +259,9 @@ bool CBOINCGUIApp::OnInit() {
         }
     }
 
+#ifdef SANDBOX
     if (!success) iErrorCode = -1016;
+#endif
 
     // Initialize the BOINC Diagnostics Framework
     int dwDiagnosticsFlags =
@@ -232,7 +269,7 @@ bool CBOINCGUIApp::OnInit() {
         BOINC_DIAG_HEAPCHECKENABLED |
         BOINC_DIAG_MEMORYLEAKCHECKENABLED |
 #endif
-        BOINC_DIAG_DUMPCALLSTACKENABLED | 
+        BOINC_DIAG_DUMPCALLSTACKENABLED |
         BOINC_DIAG_PERUSERLOGFILES |
         BOINC_DIAG_REDIRECTSTDERR |
         BOINC_DIAG_REDIRECTSTDOUT |
@@ -257,12 +294,25 @@ bool CBOINCGUIApp::OnInit() {
     wxASSERT(m_pLocale);
 
     //
-    if (m_strISOLanguageCode.IsEmpty()) {
-        iDesiredLanguageCode = wxLANGUAGE_DEFAULT;
-        m_pLocale->Init(iDesiredLanguageCode);
+    if (!m_bUseDefaultLocale && !m_strISOLanguageCode.IsEmpty()) {
+        const wxLanguageInfo* pLI = wxLocale::FindLanguageInfo(m_strISOLanguageCode);
+        if (pLI) {
+            iDesiredLanguageCode = pLI->Language;
+        }
+    }
+
+#ifdef __WXMAC__
+    // wxLocale::Init(wxLANGUAGE_DEFAULT) does not work correctly
+    // on the Mac so we must use wxUILocale::GetSystemLanguage().
+    if (m_bUseDefaultLocale || (iDesiredLanguageCode == wxLANGUAGE_DEFAULT)) {
+        iDesiredLanguageCode = wxUILocale::GetSystemLanguage();
+        m_strISOLanguageCode = wxLocale::GetLanguageCanonicalName(iDesiredLanguageCode);
+    }
+#endif
+
+    m_pLocale->Init(iDesiredLanguageCode);
+    if (iDesiredLanguageCode == wxLANGUAGE_DEFAULT) {
         m_strISOLanguageCode = m_pLocale->GetCanonicalName();
-    } else {
-        m_pLocale->Init(wxLocale::FindLanguageInfo(m_strISOLanguageCode)->Language);
     }
 
     // Look for the localization files by absolute and relative locations.
@@ -310,7 +360,7 @@ bool CBOINCGUIApp::OnInit() {
     //
     char path_to_error[MAXPATHLEN];
     path_to_error[0] = '\0';
-    
+
     if (!iErrorCode) {
         iErrorCode = check_security(
             g_use_sandbox, true, path_to_error, sizeof(path_to_error)
@@ -355,14 +405,14 @@ bool CBOINCGUIApp::OnInit() {
                 strDialogMessage += wxString::FromUTF8(path_to_error);
             }
             strDialogMessage += _(")");
-            
+
             fprintf(stderr, "%s\n", (const char*)strDialogMessage.utf8_str());
         }
 
         wxMessageDialog* pDlg = new wxMessageDialog(
-                                    NULL, 
-                                    strDialogMessage, 
-                                    m_pSkinManager->GetAdvanced()->GetApplicationName(), 
+                                    NULL,
+                                    strDialogMessage,
+                                    m_pSkinManager->GetAdvanced()->GetApplicationName(),
                                     wxOK
                                     );
 
@@ -378,9 +428,9 @@ bool CBOINCGUIApp::OnInit() {
 #ifdef __WXMSW__
     // Perform any last minute checks that should keep the manager
     // from starting up.
-    wxString strRebootPendingFile = 
+    wxString strRebootPendingFile =
         GetRootDirectory() + wxFileName::GetPathSeparator() + wxT("RebootPending.txt");
-    
+
     if (wxFile::Exists(strRebootPendingFile)) {
         wxMessageDialog dialog(
             NULL,
@@ -411,7 +461,14 @@ bool CBOINCGUIApp::OnInit() {
     // Detect if BOINC Manager is already running, if so, bring it into the
     // foreground and then exit.
     if (DetectDuplicateInstance()) {
-            return false;
+      wxMessageDialog dialog(
+          NULL,
+          _("Another instance of BOINC Manager is already running."),
+          _("BOINC Manager"),
+          wxOK|wxICON_WARNING
+      );
+      dialog.ShowModal();
+      return false;
     }
 
     // Initialize the main document
@@ -432,7 +489,7 @@ bool CBOINCGUIApp::OnInit() {
 #ifndef __WXGTK__
     // Initialize the task bar icon
 	m_pTaskBarIcon = new CTaskBarIcon(
-        m_pSkinManager->GetAdvanced()->GetApplicationName(), 
+        m_pSkinManager->GetAdvanced()->GetApplicationName(),
         m_pSkinManager->GetAdvanced()->GetApplicationIcon(),
         m_pSkinManager->GetAdvanced()->GetApplicationDisconnectedIcon(),
         m_pSkinManager->GetAdvanced()->GetApplicationSnoozeIcon()
@@ -444,7 +501,7 @@ bool CBOINCGUIApp::OnInit() {
 #endif // __WXGTK__
 #ifdef __WXMAC__
     m_pMacDockIcon = new CTaskBarIcon(
-        m_pSkinManager->GetAdvanced()->GetApplicationName(), 
+        m_pSkinManager->GetAdvanced()->GetApplicationName(),
         m_pSkinManager->GetAdvanced()->GetApplicationIcon(),
         m_pSkinManager->GetAdvanced()->GetApplicationDisconnectedIcon(),
         m_pSkinManager->GetAdvanced()->GetApplicationSnoozeIcon()
@@ -455,7 +512,7 @@ bool CBOINCGUIApp::OnInit() {
 
     // Startup the System Idle Detection code
     IdleTrackerAttach();
-    
+
     // Show the UI
     SetActiveGUI(m_iGUISelected, m_bGUIVisible);
 
@@ -469,14 +526,14 @@ bool CBOINCGUIApp::OnInit() {
             m_pFrame->Raise();
         }
     }
-    
+
     return true;
 }
 
 #ifdef __WXMAC__
 // We can "show" (unhide) the main window when the
 // application is hidden and it won't be visible.
-// If we don't do this under wxCocoa 3.0, the Dock 
+// If we don't do this under wxCocoa 3.0, the Dock
 // icon will bounce (as in notification) when we
 // click on our menu bar icon.
 // But wxFrame::Show(true) makes the application
@@ -487,13 +544,13 @@ bool CBOINCGUIApp::OnInit() {
 // loop is running, so this is called from
 // CBOINCBaseFrame::OnPeriodicRPC() at the first
 // firing of ID_PERIODICRPCTIMER.
-// 
+//
 void CBOINCGUIApp::OnFinishInit() {
     if (!m_bGUIVisible) {
         HideThisApp();
-    
+
         m_pFrame->wxWindow::Show();
-        
+
         if (m_pEventLog) {
             m_pEventLog->wxWindow::Show();
         }
@@ -587,9 +644,13 @@ void CBOINCGUIApp::SaveState() {
         m_pConfig->Write(wxT("Skin"), m_pSkinManager->GetSelectedSkin());
     }
     m_pConfig->Write(wxT("LanguageISO"), m_strISOLanguageCode);
+    m_pConfig->Write(wxT("UseDefaultLocale"), m_bUseDefaultLocale);
     m_pConfig->Write(wxT("AutomaticallyShutdownClient"), m_iShutdownCoreClient);
     m_pConfig->Write(wxT("DisplayShutdownClientDialog"), m_iDisplayExitDialog);
     m_pConfig->Write(wxT("DisplayShutdownConnectedClientDialog"), m_iDisplayShutdownConnectedClientDialog);
+#ifdef __WXMAC__
+    m_pConfig->Write(wxT("HideMenuBarIcon"), m_iHideMenuBarIcon);
+#endif
     m_pConfig->Write(wxT("DisableAutoStart"), m_iBOINCMGRDisableAutoStart);
     m_pConfig->Write(wxT("RunDaemon"), m_bRunDaemon);
 }
@@ -665,7 +726,7 @@ bool CBOINCGUIApp::OnCmdLineParsed(wxCmdLineParser &parser) {
     if (m_strBOINCMGRDataDirectory.Last() != '/') {
         m_strBOINCMGRDataDirectory.Append('/');
     }
-#endif    
+#endif
 
     if (parser.Found(wxT("namehost"), &m_strHostNameArg)) {
         hostNameSpecified = true;
@@ -682,7 +743,7 @@ bool CBOINCGUIApp::OnCmdLineParsed(wxCmdLineParser &parser) {
     } else {
         m_iRPCPortArg = GUI_RPC_PORT;
     }
-    
+
     if (parser.Found(wxT("password"), &m_strPasswordArg)) {
         passwordSpecified = true;
     } else {
@@ -737,7 +798,7 @@ void CBOINCGUIApp::DetectExecutableName() {
 
     // change the current directory to the boinc install directory
     GetModuleFileName(NULL, szPath, (sizeof(szPath)/sizeof(TCHAR)));
-		
+
     TCHAR *pszProg = _tcsrchr(szPath, '\\');
     if (pszProg) {
         pszProg++;
@@ -768,7 +829,7 @@ void CBOINCGUIApp::DetectRootDirectory() {
 
     // change the current directory to the boinc install directory
     GetModuleFileName(NULL, szPath, (sizeof(szPath)/sizeof(TCHAR)));
-		
+
     TCHAR *pszProg = _tcsrchr(szPath, '\\');
     if (pszProg) {
         szPath[pszProg - szPath + 1] = 0;
@@ -809,9 +870,9 @@ void CBOINCGUIApp::DetectDataDirectory() {
 
     // change the current directory to the boinc data directory if it exists
 	lReturnValue = RegOpenKeyEx(
-        HKEY_LOCAL_MACHINE, 
+        HKEY_LOCAL_MACHINE,
         _T("SOFTWARE\\Space Sciences Laboratory, U.C. Berkeley\\BOINC Setup"),
-		0, 
+		0,
         KEY_READ,
         &hkSetupHive
     );
@@ -831,7 +892,7 @@ void CBOINCGUIApp::DetectDataDirectory() {
             (*lpszValue) = NULL;
 
             // Now get the data
-            lReturnValue = RegQueryValueEx( 
+            lReturnValue = RegQueryValueEx(
                 hkSetupHive,
                 _T("DATADIR"),
                 NULL,
@@ -844,13 +905,13 @@ void CBOINCGUIApp::DetectDataDirectory() {
             // We need to get the size of the buffer needed
             dwSize = 0;
             lReturnValue = ExpandEnvironmentStrings(lpszValue, NULL, dwSize);
-   
+
             if (lReturnValue) {
                 // Make the buffer big enough for the expanded string
                 lpszExpandedValue = (LPTSTR) malloc(lReturnValue*sizeof(TCHAR));
                 (*lpszExpandedValue) = NULL;
                 dwSize = lReturnValue;
-   
+
                 ExpandEnvironmentStrings(lpszValue, lpszExpandedValue, dwSize);
 
                 // Store the root directory for later use.
@@ -879,23 +940,113 @@ void CBOINCGUIApp::DetectDataDirectory() {
 
 
 void CBOINCGUIApp::InitSupportedLanguages() {
-    wxInt32               iIndex = 0;
-    const wxLanguageInfo* liLanguage = NULL;
+    m_astrLanguages.clear();
 
-    // Prepare the array
-    m_astrLanguages.Insert(wxEmptyString, 0, wxLANGUAGE_USER_DEFINED+1);
-
-    // These are just special tags so deal with them in a special way
-    m_astrLanguages[wxLANGUAGE_DEFAULT]                    = _("(Automatic Detection)");
-    m_astrLanguages[wxLANGUAGE_UNKNOWN]                    = _("(Unknown)");
-    m_astrLanguages[wxLANGUAGE_USER_DEFINED]               = _("(User Defined)");
-
-    for (iIndex = 0; iIndex <= wxLANGUAGE_USER_DEFINED; iIndex++) {
-        liLanguage = wxLocale::GetLanguageInfo(iIndex);
-        if (liLanguage) {
-            m_astrLanguages[iIndex] = liLanguage->Description;
+    // Find available translations
+    std::vector<const wxLanguageInfo*> availableTranslations;
+    // English is a special case:
+    //  - it's guaranteed to be available because it's compiled in
+    //  - it must be added to the list even though we don't expect to find a translation for it
+    const wxLanguageInfo* pLIen = wxLocale::GetLanguageInfo(wxLANGUAGE_ENGLISH);
+    if (pLIen) {
+        availableTranslations.push_back(pLIen);
+    }
+    // Now fill in the rest from the available message catalogs
+    const wxTranslations* pTranslations = wxTranslations::Get();
+    if (pTranslations) {
+        wxArrayString langCodes = pTranslations->GetAvailableTranslations(wxT("BOINC-Manager"));
+        for (const wxString& langCode : langCodes) {
+            if (langCode == wxT("en")) continue;
+            const wxLanguageInfo* pLI = wxLocale::FindLanguageInfo(langCode);
+            if (pLI) {
+                availableTranslations.push_back(pLI);
+            }
         }
     }
+
+    // Synthesize labels to be used in the options dialog
+    //
+    // As we are building strings that potentially contain both left-to-right and
+    // right-to-left text, we must insert direction markers to ensure the layout is
+    // correct. Otherwise strange things happen - particularly when the strings contain
+    // parentheses, which can end up in the wrong place and pointing the wrong way.
+    // The usage here has been determined largely by trial and error, and may not be
+    // strictly correct...
+    const wxString LRM = L'\x200E'/*LEFT-TO-RIGHT MARK*/;
+    const wxString RLM = L'\x200F'/*RIGHT-TO-LEFT MARK*/;
+    const wxLanguageInfo* pLIui = wxLocale::FindLanguageInfo(GetISOLanguageCode());
+    wxLayoutDirection uiLayoutDirection = pLIui ? pLIui->LayoutDirection : wxLayout_Default;
+    GUI_SUPPORTED_LANG newItem;
+
+    // CDlgOptions depends on "Auto" being the first item in the list
+    // if
+    newItem.Language = wxLANGUAGE_DEFAULT;
+    wxString strAutoEnglish = wxT("(Automatic Detection)");
+    wxString strAutoTranslated = wxGetTranslation(strAutoEnglish);
+    newItem.Label = strAutoTranslated;
+    if (strAutoTranslated != strAutoEnglish) {
+        if (uiLayoutDirection == wxLayout_RightToLeft) {
+            newItem.Label += RLM;
+        } else if (uiLayoutDirection == wxLayout_LeftToRight) {
+            newItem.Label += LRM;
+        }
+        newItem.Label += wxT(" ");
+        if (uiLayoutDirection == wxLayout_RightToLeft) {
+            newItem.Label += RLM;
+        }
+        newItem.Label += LRM + strAutoEnglish + LRM;
+    }
+    m_astrLanguages.push_back(newItem);
+
+    // Add known locales to the list
+    for (int langID = wxLANGUAGE_UNKNOWN+1; langID < wxLANGUAGE_USER_DEFINED; ++langID) {
+        const wxLanguageInfo* pLI = wxLocale::GetLanguageInfo(langID);
+        if (pLI == NULL) continue;
+        wxString lang_region = pLI->CanonicalName.BeforeFirst('@');
+        wxString lang = lang_region.BeforeFirst('_');
+        wxString script = pLI->CanonicalName.AfterFirst('@');
+        wxString lang_script = lang;
+        if (!script.empty()) {
+            lang_script += wxT("@") + script;
+        }
+        std::vector<const wxLanguageInfo*>::const_iterator foundit = availableTranslations.begin();
+        while (foundit != availableTranslations.end()) {
+            const wxLanguageInfo* pLIavail = *foundit;
+            if (pLIavail->CanonicalName == lang_script ||
+                pLIavail->CanonicalName == pLI->CanonicalName) {
+                break;
+            }
+            ++foundit;
+        }
+        // If we don't have a translation, don't add to the list -
+        // unless the locale has been explicitly selected by the user
+        // (setting migrated from an earlier version, or manually configured)
+        if (foundit == availableTranslations.end() && pLI != pLIui) continue;
+        newItem.Language = langID;
+#if wxCHECK_VERSION(3,1,6)
+        if (pLI->DescriptionNative != pLI->Description &&
+            !pLI->DescriptionNative.empty()) {
+            // The "NativeName (EnglishName)" format of the label matches that used
+            // for Web sites [language_select() in html/inc/language_names.inc]
+            newItem.Label = pLI->DescriptionNative;
+            if (pLI->LayoutDirection == wxLayout_RightToLeft) {
+                newItem.Label += RLM;
+            } else if (pLI->LayoutDirection == wxLayout_LeftToRight) {
+                newItem.Label += LRM;
+            }
+            newItem.Label += wxT(" ");
+            if (uiLayoutDirection == wxLayout_RightToLeft) {
+                newItem.Label += RLM;
+            }
+            newItem.Label += LRM + wxT("(") + pLI->Description + wxT(")") + LRM;
+        } else {
+            newItem.Label = pLI->Description + LRM;
+        }
+#else
+        newItem.Label = pLI->Description + LRM;
+#endif
+        m_astrLanguages.push_back(newItem);
+    }
 }
 
 
@@ -931,7 +1082,7 @@ void CBOINCGUIApp::OnActivateApp(wxActivateEvent& event) {
 #endif
         {
             bool keepEventLogInFront = m_bEventLogWasActive;
-            
+
             if (m_pEventLog && !m_pEventLog->IsIconized() && !keepEventLogInFront) {
                 m_pEventLog->Raise();
             }
@@ -946,17 +1097,17 @@ void CBOINCGUIApp::OnActivateApp(wxActivateEvent& event) {
     }
 
     event.Skip();
-    
+
     m_bProcessingActivateAppEvent = false;
 }
 
 
 void CBOINCGUIApp::OnRPCFinished( CRPCFinishedEvent& event ) {
     CMainDocument*      pDoc = wxGetApp().GetDocument();
-   
+
     wxASSERT(pDoc);
     wxASSERT(wxDynamicCast(pDoc, CMainDocument));
-    
+
     pDoc->OnRPCComplete(event);
 }
 
@@ -1035,7 +1186,7 @@ void CBOINCGUIApp::OnEventLogClose() {
     }
 }
 
-    
+
 // The skin has changed and all UI elements need to reload their bitmaps.
 //
 void CBOINCGUIApp::FireReloadSkin() {
@@ -1083,7 +1234,7 @@ bool CBOINCGUIApp::SetActiveGUI(int iGUISelection, bool bShowWindow) {
             m_pConfig->Read(wxT("YPos"), &iTop, 30);
             m_pConfig->Read(wxT("XPos"), &iLeft, 30);
 
-            // We don't save Simple View's width & height since it's 
+            // We don't save Simple View's width & height since it's
             // window is not resizable, so don't try to read them
 #ifdef __WXMAC__
 //            m_pConfig->Read(wxT("Width"), &iWidth, 409);
@@ -1125,7 +1276,7 @@ bool CBOINCGUIApp::SetActiveGUI(int iGUISelection, bool bShowWindow) {
         if (BOINC_ADVANCEDGUI == iGUISelection) {
             // Initialize the advanced gui window
             pNewFrame = new CAdvancedFrame(
-                m_pSkinManager->GetAdvanced()->GetApplicationName(), 
+                m_pSkinManager->GetAdvanced()->GetApplicationName(),
                 m_pSkinManager->GetAdvanced()->GetApplicationIcon(),
                 wxPoint(iLeft, iTop),
                 wxSize(iWidth, iHeight)
@@ -1133,7 +1284,7 @@ bool CBOINCGUIApp::SetActiveGUI(int iGUISelection, bool bShowWindow) {
         } else {
             // Initialize the simple gui window
             pNewFrame = new CSimpleFrame(
-                m_pSkinManager->GetAdvanced()->GetApplicationName(), 
+                m_pSkinManager->GetAdvanced()->GetApplicationName(),
                 m_pSkinManager->GetAdvanced()->GetApplicationIcon(),
                 wxPoint(iLeft, iTop),
                 wxSize(iWidth, iHeight)
@@ -1148,7 +1299,7 @@ bool CBOINCGUIApp::SetActiveGUI(int iGUISelection, bool bShowWindow) {
             // Store the new frame for future use
             m_pFrame = pNewFrame;
 
-            // Hide the old one if it exists.  We must do this 
+            // Hide the old one if it exists.  We must do this
             // after updating m_pFrame to prevent Mac OSX from
             // hiding the application
             if (pOldFrame) pOldFrame->Hide();
@@ -1165,7 +1316,7 @@ bool CBOINCGUIApp::SetActiveGUI(int iGUISelection, bool bShowWindow) {
         }
     }
 
-    // Show the new frame if needed 
+    // Show the new frame if needed
     if (!m_bProcessingActivateAppEvent) {
         if (m_pFrame && bShowWindow) {
             if (m_pEventLog && !m_pEventLog->IsIconized()) {
@@ -1209,18 +1360,18 @@ int CBOINCGUIApp::ConfirmExit() {
     wxASSERT(pSkinAdvanced);
     wxASSERT(wxDynamicCast(pDoc, CMainDocument));
     wxASSERT(wxDynamicCast(pSkinAdvanced, CSkinAdvanced));
-    
+
     pDoc->GetConnectedComputerName(strConnectedCompter);
     if (!pDoc->IsComputerNameLocal(strConnectedCompter)) {
         // Don't shut down remote clients on Manager exit
         return 1;
     }
 
-    // Don't run confirmation dialog if logging out or shutting down Mac, 
+    // Don't run confirmation dialog if logging out or shutting down Mac,
     // or if emergency exit from AsyncRPCDlg
     if (s_bSkipExitConfirmation) return 1;
 
-    // Don't run confirmation dialog if second instance of Manager 
+    // Don't run confirmation dialog if second instance of Manager
     if (IsMgrMultipleInstance()) return 1;
 
     if (!m_iDisplayExitDialog) {
@@ -1270,16 +1421,16 @@ int CBOINCGUIApp::ConfirmExit() {
 }
 
 
-// Use this instead of wxMessageBox from all tab Views to suppress 
+// Use this instead of wxMessageBox from all tab Views to suppress
 // Periodic RPCs.  See comment in CMainDocument::RunPeriodicRPCs()
 // for a fuller explanation.
 int CBOINCGUIApp::SafeMessageBox(const wxString& message, const wxString& caption, long style,
                  wxWindow *parent, int x, int y )
 {
     int retval;
-    
+
     m_bSafeMessageBoxDisplayed++;
-    
+
     retval = wxMessageBox(message, caption, style, parent, x, y);
 
     m_bSafeMessageBoxDisplayed--;
@@ -1295,7 +1446,7 @@ int CBOINCGUIApp::SafeMessageBox(const wxString& message, const wxString& captio
 ///
 /// @return
 ///  true if the current process is visible, otherwise false.
-/// 
+///
 bool CBOINCGUIApp::IsApplicationVisible() {
     return false;
 }
@@ -1332,13 +1483,13 @@ bool CBOINCGUIApp::ShowNotifications() {
 
 bool CBOINCGUIApp::IsModalDialogDisplayed() {
     if (m_bSafeMessageBoxDisplayed) return true;
-    
-    // Search for the dialog by ID since all of BOINC Manager's 
+
+    // Search for the dialog by ID since all of BOINC Manager's
     // dialog IDs are 10000.
     if (wxDynamicCast(wxWindow::FindWindowById(ID_ANYDIALOG), wxDialog)) {
         return true;
     }
-    
+
     if (m_pDocument) {
         if (m_pDocument->WaitingForRPC()) {
             return true;
@@ -1358,14 +1509,14 @@ int CBOINCGUIApp::FilterEvent(wxEvent &event) {
     theEventType = event.GetEventType();
 
     if (m_pDocument->WaitingForRPC()) {
-        // If in RPC Please Wait dialog, reject all command 
-        // and timer events except: 
+        // If in RPC Please Wait dialog, reject all command
+        // and timer events except:
         //  - RPC Finished
         //  - those for that dialog or its children
         //  - Open Manager menu item from system tray icon
 
         if ((theEventType == wxEVT_COMMAND_MENU_SELECTED) && (event.GetId() == wxID_OPEN)) {
-            return -1;        
+            return -1;
         }
 
         theRPCWaitDialog = m_pDocument->GetRPCWaitDialog();
@@ -1377,7 +1528,7 @@ int CBOINCGUIApp::FilterEvent(wxEvent &event) {
         }
         // Continue with rest of filtering below
     } else {
-        // Do limited filtering if shutting down to allow RPC 
+        // Do limited filtering if shutting down to allow RPC
         // completion events but not events which start new RPCs
         if (!m_bFilterEvents) return -1;
     }
@@ -1386,17 +1537,17 @@ int CBOINCGUIApp::FilterEvent(wxEvent &event) {
     if (event.IsCommandEvent()) {
         return false;
     }
-    
+
     if (theEventType == wxEVT_TIMER) {
         return false;
     }
-    
+
 #ifdef __WXMSW__
     if (theEventType == wxEVT_TASKBAR_MOVE) {
         return false;
     }
 #endif
-   
+
     return -1;
 }
 
