@@ -1,6 +1,6 @@
 // This file is part of BOINC.
 // http://boinc.berkeley.edu
-// Copyright (C) 2008 University of California
+// Copyright (C) 2023 University of California
 //
 // BOINC is free software; you can redistribute it and/or modify it
 // under the terms of the GNU Lesser General Public License
@@ -45,6 +45,7 @@ CViewResources::CViewResources()
 CViewResources::CViewResources(wxNotebook* pNotebook) :
 	CBOINCBaseView(pNotebook)
 {
+    bool isDarkMode = wxGetApp().GetIsDarkMode();
 	m_BOINCwasEmpty=false;
 
 	wxGridSizer* itemGridSizer = new wxGridSizer(2, 0, 3);
@@ -58,7 +59,7 @@ CViewResources::CViewResources(wxNotebook* pNotebook) :
     m_pieCtrlTotal->SetTransparent(true);
     m_pieCtrlTotal->SetHorLegendBorder(10);
     m_pieCtrlTotal->SetLabelFont(*wxSWISS_FONT);
-    m_pieCtrlTotal->SetLabelColour(wxColour(0,0,0));
+    m_pieCtrlTotal->SetLabelColour(isDarkMode ? *wxWHITE : *wxBLACK);
     m_pieCtrlTotal->SetLabel(_("Total disk usage"));
 
     // initialize pie control
@@ -70,7 +71,7 @@ CViewResources::CViewResources(wxNotebook* pNotebook) :
 	m_pieCtrlTotal->m_Series.Add(CtrlTotalPart);
 	m_pieCtrlTotal->Refresh();
 
-    
+
     // create pie chart ctrl for BOINC disk usage
 	m_pieCtrlBOINC = new wxPieCtrl(this, ID_PIECTRL_RESOURCEUTILIZATIONVIEW, wxDefaultPosition, wxDefaultSize);
 	wxASSERT(m_pieCtrlBOINC);
@@ -79,7 +80,7 @@ CViewResources::CViewResources(wxNotebook* pNotebook) :
     m_pieCtrlBOINC->SetTransparent(true);
     m_pieCtrlBOINC->SetHorLegendBorder(10);
     m_pieCtrlBOINC->SetLabelFont(*wxSWISS_FONT);
-    m_pieCtrlBOINC->SetLabelColour(wxColour(0,0,0));
+    m_pieCtrlTotal->SetLabelColour(isDarkMode ? *wxWHITE : *wxBLACK);
     m_pieCtrlBOINC->SetLabel(_("Disk usage by BOINC projects"));
 
     // initialize pie control
@@ -89,6 +90,8 @@ CViewResources::CViewResources(wxNotebook* pNotebook) :
 	CtrlBOINCPart.SetValue(1);
 	CtrlBOINCPart.SetColour(wxColour(255,255,255));
 	m_pieCtrlBOINC->m_Series.Add(CtrlBOINCPart);
+	m_pieCtrlTotal->SetLayoutDirection(wxLayout_LeftToRight);
+	m_pieCtrlBOINC->SetLayoutDirection(wxLayout_LeftToRight);
 	m_pieCtrlBOINC->Refresh();
 
     //init the flexGrid
@@ -170,6 +173,7 @@ void CViewResources::OnListRender( wxTimerEvent& WXUNUSED(event) ) {
     wxString diskspace;
 	static double project_total=0.0;
 	unsigned int i;
+    bool isDarkMode = wxGetApp().GetIsDarkMode();
 
     wxASSERT(pDoc);
     wxASSERT(wxDynamicCast(pDoc, CMainDocument));
@@ -224,11 +228,12 @@ void CViewResources::OnListRender( wxTimerEvent& WXUNUSED(event) ) {
 			wxPiePart part;
             part.SetLabel(_("no projects: 0 bytes used"));
 			part.SetValue(1);
-			part.SetColour(wxColour(0,0,0));
+			part.SetColour(isDarkMode ? wxColour(255, 255, 255) : wxColour(0,0,0));
 			m_pieCtrlBOINC->m_Series.Add(part);
 			m_pieCtrlBOINC->Refresh();
 			m_BOINCwasEmpty=true;
 			refreshBOINC=true;
+			project_total = 0;
         }
 	}
 
@@ -258,7 +263,8 @@ void CViewResources::OnListRender( wxTimerEvent& WXUNUSED(event) ) {
         FormatDiskSpace(boinc_total, diskspace);
         part.SetLabel(_("used by BOINC: ") + diskspace);
 		part.SetValue(boinc_total);
-		part.SetColour(wxColour(0,0,0));
+        part.SetColour(isDarkMode ? wxColour(255, 255, 255) : wxColour(0,0,0));
+		part.SetColour(isDarkMode ? *wxWHITE : *wxBLACK);
 		m_pieCtrlTotal->m_Series.Add(part);
 
         if (pDoc->disk_usage.d_allowed > 0) {
@@ -268,7 +274,7 @@ void CViewResources::OnListRender( wxTimerEvent& WXUNUSED(event) ) {
             FormatDiskSpace(avail, diskspace);
             part.SetLabel(_("free, available to BOINC: ") + diskspace);
             part.SetValue(avail == 0 ? 1 : avail);
-            part.SetColour(wxColour(128, 128, 128));
+            part.SetColour(isDarkMode ? wxColour(108, 108, 108) : wxColour(128, 128, 128));
             m_pieCtrlTotal->m_Series.Add(part);
 
             double not_avail = free - avail;
@@ -276,7 +282,7 @@ void CViewResources::OnListRender( wxTimerEvent& WXUNUSED(event) ) {
 		        FormatDiskSpace(not_avail, diskspace);
                 part.SetLabel(_("free, not available to BOINC: ") + diskspace);
 		        part.SetValue(not_avail);
-		        part.SetColour(wxColour(238,238,238));
+		        part.SetColour(isDarkMode ? wxColour(172,172,172) : wxColour(238,238,238));
 		        m_pieCtrlTotal->m_Series.Add(part);
             }
         } else {
@@ -286,7 +292,7 @@ void CViewResources::OnListRender( wxTimerEvent& WXUNUSED(event) ) {
 		    FormatDiskSpace(free, diskspace);
             part.SetLabel(_("free: ") + diskspace);
 		    part.SetValue(free);
-		    part.SetColour(wxColour(238,238,238));
+            part.SetColour(isDarkMode ? wxColour(172,172,172) : wxColour(238,238,238));
 		    m_pieCtrlTotal->m_Series.Add(part);
         }
 
@@ -295,7 +301,7 @@ void CViewResources::OnListRender( wxTimerEvent& WXUNUSED(event) ) {
 		FormatDiskSpace(used_by_others, diskspace);
         part.SetLabel(_("used by other programs: ") + diskspace);
 		part.SetValue(used_by_others);
-		part.SetColour(wxColour(192,192,192));
+		part.SetColour(isDarkMode ? wxColour(140,140,140) : wxColour(192,192,192));
 		m_pieCtrlTotal->m_Series.Add(part);
 		m_pieCtrlTotal->Refresh();
 	}
